#!/usr/bin/env bash

license() {
    cat << EOF
    ■The contents of this file are subject to the Common Public Attribution
    License Version 1.0 (the ■License■); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    https://opensource.org/licenses/CPAL-1.0. The License is based on the Mozilla Public License Version
    1.1 but Sections 14 and 15 have been added to cover use of software over a
    computer network and provide for limited attribution for the Original
    Developer. In addition, Exhibit A has been modified to be consistent with
    Exhibit B.

    Software distributed under the License is distributed on an ■AS IS■ basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is audiogame manager.

    The Original Developer is not the Initial Developer and is . If
    left blank, the Original Developer is the Initial Developer.

    The Initial Developer of the Original Code is Billy "Storm Dragon" Wolfe. All portions of
    the code written by Billy Wolfe are Copyright (c) 2020, 2024. All Rights
    Reserved.

    Contributor Michael Taboada.

    Attribution Copyright Notice: Audiogame manager copyright 2020 Storm Dragon. All rights reserved.

    Attribution Phrase (not exceeding 10 words): A Stormux project

    Attribution URL: https://stormgames.wolfe.casa

    Graphic Image as provided in the Covered Code, if any.

    Display of Attribution Information is required in Larger
    Works which are defined in the CPAL as a work which combines Covered Code
    or portions thereof with code not governed by the terms of the CPAL.
EOF
}

# Dialog accessibility
export DIALOGOPTS='--no-lines --visit-items'


# Alerts, for when user needs to read something.
alert() {
    play -qnV0 synth 3 pluck D3 pluck A3 pluck D4 pluck F4 pluck A4 delay 0 .1 .2 .3 .4 remix - chorus 0.9 0.9 38 0.75 0.3 0.5 -t
    echo
    read -rp "Press enter to continue." continue
}

# Check for latest news
check_news() {
    # For use by update scripts that want to source functions in this file.
    [[ "$agmNoLaunch" == "true" ]] && return
    trap return INT
    # url for news file
    local newsFile="https://stormgames.wolfe.casa/media/agm.ogg"
    local newsPath="${configFile%/*.conf}/.news"
    local newsTag="$(curl --connect-timeout 5 -sI "$newsFile" | grep -i '^etag: "' | cut -d '"' -f2)"
    if [[ -z "${newsTag}" ]]; then
        return
    fi
    local newsOldTag="$(cat "$newsPath" 2> /dev/null)"
    if [[ "$newsTag" != "$newsOldTag" ]]; then
        dialog --yes-label 'Play' \
            --no-label 'Later' \
            --backtitle 'Audiogame Manager News' \
            --yesno 'Audiogame manager news is available. Please use left and right arrows to navigate and enter to confirm.' -1 -1 || return
        sox -qV0 "$newsFile" -d  &> /dev/null
        echo -n "$newsTag" > "$newsPath"
    fi
}

# Automatic update function
update() {
    if ! [[ -d ".git" ]]; then
        return
    fi
    local url="$(git ls-remote --get-url)"
    if [[ "$url" =~ ^ssh://|git@|gitea@ ]] || [[ -z "$url" ]]; then
        return
    fi
    git remote update &> /dev/null
    local upstream='@{u}'
    local home="$(git rev-parse @)"
    local remote="$(git rev-parse "$upstream")"
if [[ "$home" == "$remote" ]]; then
    return
fi
    dialog --backtitle "Audiogame Manager" \
        --yesno "Updates are available. Would you like to update now?" -1 -1 --stdout || return
    git pull
    exit $?
}

# Function to open urls across OS.
open_url() {
    if [[ "$(uname)" == "Darwin" ]]; then
        open "${*}" 2> /dev/null
    else
        xdg-open "${*}" 2> /dev/null
    fi
}

# Create desktop launcher file
desktop_launcher() {
    local desktopFile="${HOME}/audiogame-manager.desktop"
    if [[ -e "${desktopFile}" ]]; then
        echo "the file ${desktopFile} exists. Cannot create the launcher."
        exit 1
    fi
    local dotDesktop
    local terminal
    # Try to find an accessible terminal
    for i in mate-terminal lxterminal terminator gnome-terminal ; do
        if command -v $i &> /dev/null ; then
            terminal="$i"
            break
        fi
    done
    dotDesktop=('[Desktop Entry]'
        'Name=Audiogame manager'
        'GenericName=Audiogame Manager'
        'Comment=Play audio games'
        "Exec=${terminal} -t \"Audiogame Manager\" -e \"/usr/bin/bash -c 'nohup $(readlink -e "$0") 2> /dev/null'\""
        'Terminal=false'
        'Type=Application'
        'StartupNotify=false'
        'Keywords=game;'
        'Categories=Game;'
    'Version=1.0')
    for i in "${dotDesktop[@]}" ; do
        echo "$i" >> "${desktopFile}"
    done
    desktop-file-install --dir "${HOME}/.local/share/applications" -m 755 "${desktopFile}"
    xdg-desktop-icon install ~/.local/share/applications/audiogame-manager.desktop
    rm "${desktopFile}"
    exit 0
}


# Wine configuration section

checklist() {
    declare -a errorList
    declare -a packageList
    if [[ $# -eq 0 ]]; then
        echo "Checking your system..."
        echo
    fi
    if command -v wine &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Wine is installed."
    else
        errorList+=("Critical: Wine is not installed. You will not be able to play any games.")
    fi
    packageList+=("wine")
    if command -v curl &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Curl is installed."
    else
        errorList+=("Critical: Curl is not installed. Critical functionality will not work.")
    fi
    packageList+=("curl")
    if command -v dialog &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Dialog is installed."
    else
        errorList+=("Critical: Dialog is not installed. You will not be able to install, launch, or remove any games.")
    fi
    packageList+=("dialog")
    for i in 7z cabextract unzip xz ; do
        if command -v $i &> /dev/null ; then
            [[ $# -eq 0 ]] && echo "${i^} is installed."
        else
            errorList+=("Critical: ${i^} is not installed. You will not be able to install some games or their components.")
        fi
        packageList+=("$i")
    done
    if command -v gawk &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Gawk is installed."
    else
        errorList+=("Warning: gawk is not installed. Game removal with -r will not work.")
    fi
    packageList+=("gawk")
    if command -v ocrdesktop &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Ocrdesktop is installed."
    else
        errorList+=("Warning: ocrdesktop is not installed. It can help if the installer gets stuck to figure out what is happening.")
    fi
    packageList+=("ocrdesktop")
    if command -v qjoypad &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Qjoypad is installed."
    else
        errorList+=("Warning: qjoypad is not installed. Qjoypad allows you to play keyboard only games with a gamepad.")
    fi
    packageList+=("qjoypad")
    if command -v sox &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Sox is installed."
    else
        errorList+=("Warning: Sox is not installed. Audio will not work.")
    fi
    packageList+=("sox")
    if command -v trans &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Translate-shell is installed."
    else
        errorList+=("Warning: translate-shell is not installed. Games that require translation will not be translated.")
    fi
    packageList+=("translate-shell")
    if command -v sqlite3 &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Sqlite3 is installed."
    else
        errorList+=("Warning: sqlite is not installed. Required for games that need to be translated.")
    fi
    if command -v perl &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Perl is installed."
    else
        errorList+=("Warning: perl is not installed. Required for games that need to be translated.")
    fi
    packageList+=("perl")
    packageList+=("sqlite")
    if command -v w3m &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "W3m is installed."
    else
        errorList+=("Warning: w3m is not installed. W3m is used to view game documentation.")
    fi
    packageList+=("w3m")
    if command -v xclip &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Xclip is installed."
    else
        errorList+=("Warning: Xclip is not installed. Some games may not speak or register properly.")
    fi
    packageList+=("xclip")
    if command -v xdotool &> /dev/null ; then
        [[ $# -eq 0 ]] && echo "Xdotool is installed."
    else
        errorList+=("Warning: Xdotool is not installed. Some installers may not work or may need manual intervention.")
    fi
    packageList+=("xdotool")
    # Show the results
    if [[ $# -ne 0 ]]; then
        for i in "${packageList[@]}" ; do
            echo "$i"
        done | sort
        exit 0
    fi
    if [[ ${#errorList[@]} -eq 0 ]]; then
        echo "No problems found, you are good to go."
        exit 0
    fi
    echo "Errors detected, here is a list along with the severity."
    echo "Note that errors marked critical mean that you will not be able to install and play games until they are resolved."
    for i in "${errorList[@]}" ; do
        echo "$i"
    done
    exit 0
}

clear_cache() {
    local answer
    if [[ ! -d "${cache}" ]]; then
        echo "No cache found at ${cache}."
        return
    fi
    while ! [[ "${answer,,}" =~ ^yes$|^no$ ]]; do
        echo "This will delete all contents of ${cache}. Are you sure you want to continue?"
        echo "Please type yes or no."
        echo
        read -r answer
    done
    if [[ "$answer" == "no" ]]; then
        return
    fi
    # All safety checks done. Delete the cache.
    rm -rfv "${cache}"
    echo "Cache deleted."
}

download() {
    local source=($@)
    for i in "${source[@]}" ; do
        local dest="${i##*/}"
        dest="${dest//%20/ }"
        dest="${dest#*\?filename=}"
        dest="${dest%\?*}"
        # Remove the destination file if it is empty.
        test -s "${cache}/${dest}" || rm -f "${cache}/${dest}" 2> /dev/null
        if [[ "${redownload}" == "true" ]] && [[ -e "${cache}/${dest}" ]]; then
         rm -v "${cache}/${dest}"
        fi
        # Skip if the item is in cache.
        test -e "${cache}/${dest}" && continue
        if ! curl -L4 -C - --retry 10 --output "${cache}/${dest}" "${i}" ; then
            local exit_code="$?"
            if [[ $exit_code -eq 22 ]]; then
                echo "File not found: \"$i\" (HTTP 404)"
                # Remove the empty file.
                rm -f "${cache}/${dest}"
                exit 1
            fi
        fi
        if file -b "${cache}/${dest}" | grep -q "HTML document" ; then
            echo "File not found: \"$i\" (HTML document probably 404)"
            rm -f "${cache}/${dest}"
            exit 1
        fi
    done
}

get_bottle() {
    # Handles games that use the same wine bottle
    case "${game}" in
        # Aprone (Jeremy Kaldobsky) games.
        "castaways"*) ;&
        "castaways-2"*) ;&
        "daytona-and-the-book-of-gold"*) ;&
        "dog-who-hates-toast"*) ;&
        "lunimals"*) ;&
        "paw-prints"*) ;&
        "penta-path"*) ;&
        "preludeamals"*) ;&
        "puzzle-divided"*) ;&
        "rettou"*) ;&
        "revelation"*) ;&
        "swamp"*) ;&
        "tarot-assistant"*) ;&
        "triple-triad"*)
            export WINEPREFIX="${HOME}/.local/wine/aprone" ;;
        "bg-"*) export WINEPREFIX="${HOME}/.local/wine/bg";;
        # draconis games
        "esp-pinball-classic"*) ;&
        "esp-pinball-extreme"*) ;&
        "esp-pinball-party-pack"*) ;&
        "silver-dollar"*) ;&
        "monkey-business"*) ;&
        "alien-outback"*) ;&
        "dyna-man"*) ;&
        "change-reaction"*) ;&
        "ten-pin-alley"*) export WINEPREFIX="${HOME}/.local/wine/draconis";;
        # l-works games group
        "duck-hunt"*) ;&
        "judgement-day"*) ;&
        "lockpick"*) ;&
        "pigeon-panic"*) ;&
        "super-egg-hunt"*) ;&
        "super-liam"*) ;&
        "the-great-toy-robbery"*) export WINEPREFIX="${HOME}/.local/wine/l-works";;
        # Nyanchan games group
        "bokurano-daibouken"*) ;&
        "laser-breakout"*) ;&
        "marina-break"*) ;&
        "mp5"*) ;&
        "screaming-strike-2"*) ;&
        "world-of-war"*) export WINEPREFIX="${HOME}/.local/wine/nyanchan";;
        # Oriol Gomez games group
        "bombercats"*) ;&
        "copter-mission"*) ;&
        "danger-on-the-wheel"*) ;&
        "death-on-the-road"*) ;&
        "fuck-that-bird"*) ;&
        "hammer-of-glory"*) ;&
        "insect-therapy"*) ;&
        "rhythm-rage"*) ;&
        "run-for-your-life"*) ;&
        "thief"*) ;&
        "villains-from-beyond"*) export WINEPREFIX="${HOME}/.local/wine/oriol-gomez";;
        # pbgames group
        "dark-destroyer"*) ;&
        "PBGames TMP") export WINEPREFIX="$HOME/.local/wine/pbgames" ;;
        # tunmi13 games group
        "battle-of-the-hunter"*) ;&
        "clashes-of-the-sky"*) ;&
        "challenge-of-the-horse"*) export WINEPREFIX="${HOME}/.local/wine/tunmi13";;
        # tunmi13-64bit games group
        "battlefield-2d"*) ;&
        "haunted-party"*) ;&
    "skateboarder-pro"*) export WINEPREFIX="${HOME}/.local/wine/tunmi13-64bit";;
        # Dan Z games group
        "lost"*) ;&
        "maze-craze"*) ;&
        "super-deekout"*)
            export norh="true"
            export WINEPREFIX="$HOME/.local/wine/dan-z"
        ;;
        *) export WINEPREFIX="${HOME}/.local/wine/${game%|*}";;
    esac
    # Wine version for bottles
    if [[ "$game" =~ entombed ]]; then
        install_wine "6.18" "32"
    fi
    if [[ "$game" =~ rs-games ]]; then
        install_wine "7.0" "32"
    fi
    if [[ "$game" =~ shadow-line ]]; then
        install_wine "7.7" "32"
    fi
}

get_installer() {
        trap "exit 0" SIGINT
        # If the file is in cache nothing else needs to be done.
        if [[ -f "${cache}/$1" ]]; then
            return
        fi
        # Create message for dialog.
        local message="Make sure $1 is available in either your Downloads or Desktop directory and press enter to continue."
        if [[ -n "$2" ]]; then
            message+="\n\nThe last good known URL for $game is:"
            message+="\n$2"
        fi
        if echo "$2" | xclip -selection clipboard 2> /dev/null ; then
            message+="\n\nThe URL has been copied to the clipboard."
        fi
        dialog --ok-label "Continue" \
            --backtitle "Audiogame Manager" \
            --msgbox "$message" -1 -1
        # Search the Desktop and Downloads directories for the installation file
        for i in ~/Downloads ~/Desktop ; do
            find $i -type f -name "$1" -exec cp -v {} "${cache}/" \;
        done
        # If the file is still not available abort.
        if [[ ! -f "${cache}/$1" ]]; then
            echo "couldn't find $1. Please download the file and try again."
            exit 1
        fi
}

get_steam() {
    # Arguments: $1 id of item for download, $2 url for game
    trap "exit 0" SIGINT
    echo "manual intervention required."
    alert
    dialog --backtitle "Audiogame Manager" \
        --yes-label "Continue with Steam" \
        --no-label "Install manually" \
        --extra-button \
        --extra-label "Exit" \
        --yesno "To install the game manually, place files in \"${WINEPREFIX}/drive_c/Program Files/${game}\"" -1 -1 --stdout
    case $? in
        0) echo "The next steps will install through steamcmd." ;;
        1)
            mkdir -p "${WINEPREFIX}/drive_c/Program Files/${game}"
            dialog --backtitle "Audiogame Manager" \
            --msgbox "Place game files in  \"${WINEPREFIX}/drive_c/Program Files/${game}\" and press enter to continue." -1 -1 --stdout
        return
        ;;
        *) exit 0 ;;
    esac
    # Check for steamcmd
    if ! command -v steamcmd &> /dev/null ; then
        dialog --backtitle "Audiogame Manager" \
            --infobox "This installer requires steamcmd. Please install steamcmd and try again." -1 -1
        exit 1
    fi
    # Create message for dialog.
    local message="Make sure ${game} is available in your Steam library and press enter to continue. The URL for ${game} is $2"
    if echo "$2" | xclip -selection clipboard 2> /dev/null ; then
        message+="\n\nThe URL has been copied to the clipboard."
    fi
    dialog --ok-label "Continue" \
        --backtitle "Audiogame Manager" \
        --msgbox "$message" -1 -1
    # Get Steam user name.
    steamUser="$(dialog --ok-label "Continue" \
        --backtitle "Audiogame Manager" \
        --inputbox "Please enter your Steam user name:" -1 -1 --stdout)"
    # Download the game
    mkdir -p "${WINEPREFIX}/drive_c/Program Files/${game}"
    steamcmd +@sSteamCmdForcePlatformType windows +force_install_dir "${WINEPREFIX}/drive_c/Program Files/$game" +login "$steamUser" +app_update "$1" +quit || { dialog  --backtitle "Audiogame Manager" \
        --infobox "Something went wrong. Please make sure you have a stable internet connection, and if the problem persists, contact audiogame-manager's developers." -1 -1
        exit 1; }
}

help() {
    echo "${0##*/}"
    echo "Released under the terms of the Common Public Attribution License Version 1.0"
    echo -e "This is a Stormux project: https://stormux.org\n"
    echo -e "Usage:\n"
    echo "With no arguments, open the game launcher."
    for i in "${!command[@]}" ; do
        echo "-${i/:/ <parameter>}: ${command[${i}]}"
    done | sort
    echo
    echo "Some settings that are often used can be stored in a settings.conf file."
    echo "If wanted, place it at the following location:"
    echo "${configFile%/*}/settings.conf"
    echo "The syntax is variable=\"value\""
    echo
    echo "ipfsGateway=\"https://gateway.pinata.cloud\" # Gateway to be used for ipfs downloads."
    echo "noCache=\"true\" # Do not keep downloaded items in the cache."
    echo "noqjoypad=\"true\" # Do not launch qjoypad."
    echo "norh=\"true\" # Do not install RHVoice."
    echo "redownload=\"true\" # Redownload sources, do not use the version stored in cache."
    echo "voiceName=\"voicename\" # Select the voice to be installed (default Bdl)."
    echo "defaultVoice=\"voicename\" # Select the default voice to use for the bottle, e.g. MSMike or RHVoice."
    echo "defaultRate=\"Default voice rate for the bottle, default 7, may not work in all games. Values 1-9 or A."
    echo "winedebug=\"flag(s)\" # Set wine debug flags, useful for development."
    exit 0
}

documentation() {
    if [[ "$2" == "Become a Patron" ]]; then
        return
    fi
    if [[ "$2" == "Donate" ]]; then
        return
    fi
    if ! command -v w3m &> /dev/null ; then
        echo "This feature of audiogame-manager requires w3m. Please install it before continuing."
        exit 1
    fi
    get_bottle "$1"
echo "Loading documentation, please wait..."
    # Try to find documentation based on common naming conventions.
    local gamePath="$(winepath -u "$2" 2> /dev/null)"
    gamePath="${gamePath%/*}"
    local gameDoc="$(find "$gamePath" -type f -iname 'user_manual.htm*' -or -iname 'user manual.htm*' -or -iname '*user guide.htm*' | head -1)"
    # Game name specific docs, add the name to the for loop.
    if [[ -z "$gameDoc" ]]; then
        for i in "troopanum.txt" "superdeekout.txt" scw.html ; do
            gameDoc="$(find "$gamePath" -type f -iname "$i" -or -iname 'manual.htm' | head -1)"
        done
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -path '*/Manual/index.html' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname '[A-Z]*Help.htm' -or -iname '[A-Z]*Help.html' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname 'manual.html' -or -iname 'manual.htm' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname 'en.html' -or -iname 'en.htm' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname 'readme.html' -or -iname 'readme.htm' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname 'manual.txt' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname 'readme.txt' -or -iname 'help.txt' | head -1)"
    fi
    if [[ -z "$gameDoc" ]]; then
        gameDoc="$(find "$gamePath" -type f -iname '*.url' -exec grep -i 'url=' {} \; | grep -iv 'score' | head -1)"
        gameDoc="${gameDoc#*=}"
        gameDoc="${gameDoc//[[:cntrl:]]/}"
    fi
    # Display documentation if available.
    if [[ -n "$gameDoc" ]]; then
        w3m "$gameDoc"
    else
        echo "No documentation found."
    fi
    exit 0
}

install_wine() {
    # Requires wine version, e.g. 7.7 and architecture, 32|64
    if [[ $# -ne 2 ]]; then
        exit 1
    fi
    # Figure out wineInstallationPath
    wineInstallationPath="${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/wine_$2/$1"
    export wine="${wineInstallationPath}/bin/wine"
    # If the path exists, wine should already be installed.
    # Just make sure we didn't wind up with a empty directory for some reason
    rmdir "${wineInstallationPath}" 2> /dev/null
    if [[ -d "${wineInstallationPath}" ]]; then
        return
    fi
    mkdir -p "${wineInstallationPath}" 2> /dev/null
    # This probably does not need to be cached, so download to tmp.
    installationFile="$(mktemp)"
    local v=$2
    v="${v/32/x86}"
    v="${v/64/x64}" # Probably wrong, so just a place holder.
    # If this goes wrong, bail out
    set -e
    { curl -L --output "${installationFile}" "https://www.playonlinux.com/wine/binaries/phoenicis/upstream-linux-x86/PlayOnLinux-wine-${1}-upstream-linux-${v}.tar.gz"
    tar xf "${installationFile}" -C "${wineInstallationPath}"; } | dialog --progressbox "Installing $2 bit Wine version $1." -1 -1
    set +e
}


unix2dos() {
    if [[ $# -eq 0 ]]; then
        echo "Usage: unix2dos file(s)."
        exit 1
    fi
    for file in "${@}" ; do
        sed -i 's/$/\r/' "${file}"
    done
}


winetricks() {
    # Report used packages to the winetricks maintainer so he knows they are being used.
    if ! [[ -e "${XDG_CACHE_HOME:-$HOME/.cache}/winetricks/track_usage" ]]; then
        mkdir -p "${XDG_CACHE_HOME:-$HOME/.cache}/winetricks/"
        echo "1" > "${XDG_CACHE_HOME:-$HOME/.cache}/winetricks/track_usage"
    fi
    # Download or update agm's copy of winetricks
    if [[ ! -e "${winetricksPath}/winetricks" ]]; then
        checkWinetricksUpdate="true"
        download "https://raw.githubusercontent.com/Winetricks/winetricks/master/src/winetricks"
        mv "${cache}/winetricks" "${winetricksPath}"
        chmod 755 "${winetricksPath}/winetricks"
    else
        if [[ "$checkWinetricksUpdate" != "true" ]]; then
            checkWinetricksUpdate="true"
            ${winetricksPath}/winetricks --self-update
        fi
    fi
    # Run the requested winetricks parameters
    ${winetricksPath}/winetricks "$@"
}

install_rhvoice() {
    if [[ -d "$HOME/.local/wine/${bottle}/drive_c/Program Files/Olga Yakovleva/" ]]; then
        return
    fi
    if [[ "$norh" == "true" ]]; then
        # Try to prevent the user from breaking speech
        # Also useful for games that do not work with RHVoice
        if [[ "${defaultVoice}" == "RHVoice" ]]; then
            unset defaultVoice
        fi
        return
    fi
    declare -A RHVoice=(
        [alan]="https://rhvoice.eu-central-1.linodeobjects.com/RHVoice-voice-English-Alan-v4.0.2008.15-setup.exe"
        [bdl]="https://rhvoice.eu-central-1.linodeobjects.com/RHVoice-voice-English-Bdl-v4.1.2008.15-setup.exe"
        [clb]="https://rhvoice.eu-central-1.linodeobjects.com/RHVoice-voice-English-Clb-v4.0.2008.15-setup.exe"
        [slt]="https://rhvoice.eu-central-1.linodeobjects.com/RHVoice-voice-English-Slt-v4.0.2008.15-setup.exe"
    )
    voiceName="${voiceName:-bdl}"
    voiceName="${voiceName,,}"
    if [[ "${RHVoice[${voiceName}]}" == "" ]]; then
        echo "Invalid RHVoice name specified, defaulting to Bdl."
        voiceName="bdl"
    fi
    local voiceFile="${RHVoice[${voiceName}]##*/}"
    download "${RHVoice[${voiceName}]}"
    winetricks -q win8
    echo "Installing RHVoice ${voiceName^}..."
    ${wine} "${cache}/${voiceFile}" &
    sleep 20
    ${wine}server -k
}

install_wine_bottle() {
    # 32 bit installations work best and are the default here, if you need to override it, do it in the game specific installation steps.
    export WINEARCH="${WINEARCH:-win32}"
    # Figure out if we are using a specific version of wine
    export wine="${wine:-$(command -v wine)}"
    # Set the WINE and WINESERVER environmental variables so winetricks will use the right installation.
    export WINE="${wine}"
    export WINESERVER="${wine}server"
    if [[ -z "$bottle" ]]; then
        local bottle="${game,,}"
        bottle="${bottle//[[:space:]]/-}"
        if [[ -d "$HOME/.local/wine/${bottle}" ]]; then
            echo "$HOME/.local/wine/${bottle} exists. Please remove it before running this installer."
            exit 1
        fi
    fi
    export WINEPREFIX="$HOME/.local/wine/${bottle}"
    # Arguments to the function are dependancies to be installed.
        # Get location of mono and gecko.
        monoPath="$(find /usr/share/wine/ -maxdepth 1 -type d -name mono 2> /dev/null)"
        geckoPath="$(find /usr/share/wine/ -maxdepth 1 -type d "gecko" 2> /dev/null)"
        if [[ -z "$monoPath" ]]; then
            download 'http://dl.winehq.org/wine/wine-mono/6.0.0/wine-mono-6.0.0-x86.msi'
            monoPath="${cache}/wine-mono-6.0.0-x86.msi"
        fi
        if [[ -z "$geckoPath" ]]; then
            download 'http://dl.winehq.org/wine/wine-gecko/2.40/wine_gecko-2.40-x86.msi'
            geckoPath="${cache}/wine_gecko-2.40-x86.msi"
        fi
            # This is in a brace list to pipe through dialog.
            { echo -n "Using "
            ${wine} --version
            DISPLAY="" ${wine}boot -u
            ${wine} msiexec /i z:"$monoPath" /quiet
            ${wine} msiexec /i z:"$geckoPath" /quiet
            if [[ "${*}" =~ (speechsdk|sapi) ]]; then
                install_rhvoice
            fi
            if [[ "${WINEARCH}" == "win64" ]]; then
                download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd"
            fi
            if [[ "${WINEARCH}" == "win64" ]] && [[ ! -f "${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd" ]]; then
                cp "${cache}/nvda2speechd" "${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd"
                chmod +x "${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd"
            fi
            winetricks -q isolate_home $@ ${winVer:-winxp} ${winetricksSettings}; } | dialog --progressbox "Installing wine bottle, please wait..." -1 -1
            # make it easy for game scripts to know which version of wine to use.
            echo "WINE=\"${WINE}\"" > "$HOME/.local/wine/${bottle}/agm.conf"
            echo "WINESERVER=\"${WINESERVER}\"" >> "$HOME/.local/wine/${bottle}/agm.conf"
            # If default voice is set, change it for the bottle
            if [[ ${#defaultVoice} -ge 2 ]] && [[ "${*}" =~ (speechsdk|sapi) ]]; then
                echo "Setting default voice for bottle \"${bottle}\" to \"${defaultVoice}\"."
                "${0%/*}/speech/set-voice.sh" -b "${bottle}" -r "${defaultRate:-7}" -v "${defaultVoice}"
            fi
}


# Install games
game_installer() {
    export LANG="en_US.UTF-8"
    # Try to deal with systems in other languages
    mapfile -t installedGames < <(sed -e '/^$/d' -e '/^[[:space:]]*#/d' "${configFile}" 2> /dev/null | cut -d '|' -f3)
    # Create the menu of installed games
    declare -a menuList
    for i in "${gameList[@]}" ; do
        local menuItem="$i"
        for j in "${installedGames[@]}" ; do
            if [[ "$j" == "$menuItem" ]]; then
                unset menuItem
            fi
        done
        if [[ -n "$menuItem" ]]; then
            menuList+=("$menuItem" "$menuItem")
        fi
    done
    if [[ ${#menuList[@]} -eq 0 ]]; then
        echo "All games are already installed."
        exit 0
    fi
    menuList+=("Donate" "Donate")
    menuList+=("Become a Patron" "Become a Patron")
    game="$(dialog --backtitle "Audio Game Installer" \
        --clear \
        --ok-label "Install" \
        --no-tags \
        --menu "Please select a game to install" 0 0 0 "${menuList[@]}" --stdout)"
}

# remove games
game_removal() {
    mapfile -t lines < <(sed -e '/^$/d' -e '/^[[:space:]]*#/d' "${configFile}" 2> /dev/null)
    if [[ ${#lines} -eq 0 ]]; then
        echo "No games found."
        exit 0
    fi
    # Create the menu of installed games
    declare -a menuList
    for i in "${lines[@]}" ; do
            IFS='|' read -ra gameInfo <<< "$i"
            menuList+=("${gameInfo[0]}|${gameInfo[1]}" "${gameInfo[2]}")
    done
    menuList+=("Donate" "Donate")
    menuList+=("Become a Patron" "Become a Patron")
    local game
    game="$(dialog --backtitle "Audio Game Removal" \
        --clear \
        --ok-label "Remove" \
        --no-tags \
        --menu "Please select a game to delete" 0 0 0 "${menuList[@]}" --stdout)"
    if [[ ${#game} -gt 0 ]]; then
        if [[ "$game" == "Donate" ]]; then
            open_url "https://ko-fi.com/stormux"
            exit 0
        fi
        if [[ "$game" == "Become a Patron" ]]; then
            open_url "https://2mb.games/product/2mb-patron/"
            exit 0
        fi
        local winePath="${game#*|}"
        export winePath="${winePath%\\*.exe}"
        local wineExec="${game#*|}"
        wineExec="${wineExec%|*}"
        wineExec="${wineExec##*\\}"
        # Confirm removal
        get_bottle "${game%|*}"
        # Make sure the game can be handled by remove
        if [[ "${HOME}/.local/wine/${game%|*}" == "${WINEPREFIX}" ]]; then
            read -rp "To remove the wine bottle \"${WINEPREFIX##*/}\" and all of its contents press enter. To cancel press control+c. " continue
            # kill any previous existing wineservers for this prefix in case they didn't shut down properly.
            wineserver -k
            # remove the game
            rm -rf "${WINEPREFIX}"
        else
            read -rp "This bottle \"${WINEPREFIX##*/}\" contains multiple entries, so only the game will be removed. To continue press enter. To cancel press control+c. " continue
            rm -rf "$(winepath "${winePath}")"
        fi
        # remove the launcher
        gawk -i inplace -vLine="${game//\\/\\\\}" '!index($0,Line)'  "$configFile"
        echo "The selected item has been deleted."
    fi
    exit 0
}

# kill games that are stuck
kill_game() {
    mapfile -t lines < <(sed '/^$/d' "${configFile}" 2> /dev/null)
    if [[ ${#lines} -eq 0 ]]; then
        echo "No games found."
        exit 0
    fi
    # Create the menu of installed games
    declare -a menuList
    for i in "${lines[@]}" ; do
            IFS='|' read -ra gameInfo <<< "$i"
            menuList+=("${gameInfo[0]}|${gameInfo[1]}" "${gameInfo[2]}")
    done
    menuList+=("Donate" "Donate")
    menuList+=("Become a Patron" "Become a Patron")
    local game="$(dialog --backtitle "Audio Game Killer" \
        --clear \
        --ok-label "Kill" \
        --no-tags \
        --menu "Please select a game to force stop" 0 0 0 "${menuList[@]}" --stdout)"
    if [[ ${#game} -gt 0 ]]; then
        if [[ "$game" == "Donate" ]]; then
            open_url "https://ko-fi.com/stormux"
            exit 0
        fi
        if [[ "$game" == "Become a Patron" ]]; then
            open_url "https://2mb.games/product/2mb-patron/"
            exit 0
        fi
        local winePath="${game#*|}"
        winePath="${winePath%\\*.exe}"
        local wineExec="${game#*|}"
        wineExec="${wineExec%|*}"
        wineExec="${wineExec##*\\}"
        # kill the wine server.
        get_bottle "${game%|*}"
        wineserver -k
        echo "The selected game has been stopped."
    fi
    exit 0
}

# for games that require custom scripts before launch or custom launch parameters
custom_launch_parameters() {
        if [[ "${game[0]}" == "dragon-pong" ]]; then
            "${0%/*}/speech/speak_window_title.sh" DragonPong.exe &
            pushd "$(winepath "$winePath")"
            wine "$wineExec"
            popd
            exit 0
        fi
        if [[ "${game[0]}" == "executioner's-rage" ]]; then
            find "${WINEPREFIX}/drive_c/Program Files" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        fi
        if [[ "${game[0]}" == "haunted-party" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
        if [[ "${game[0]}" == "laser-breakout" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
            "${0%/*}/speech/speak_window_title.sh" play.exe &
        fi
        if [[ "${game[0]}" == "light-battles" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
        if [[ "${game[0]}" == "mist-world" ]]; then
        "${0%/*}/speech/speak_window_title.sh" mw.exe &
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
        if [[ "${game[0]}" == "oh-shit" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
        if [[ "${game[0]}" == "bokurano-daibouken-2" ]]; then
            "${0%/*}/speech/clipboard_translator.sh" play.exe bokurano-daibouken2 &
        fi
        if [[ "${game[0]}" == "bokurano-daibouken" ]]; then
            "${0%/*}/speech/clipboard_translator.sh" play.exe bokurano-daibouken &
        fi
        if [[ "${game[0]}" =~ "bokurano-daibouken-3" ]]; then
            dictPath="$(winepath "${winePath}")"
            if [[ -r "${cache}/bk3-dict.dat" ]] && [[ ! -d "${dictPath}/dict" ]]; then
                cp "${cache}/bk3-dict.dat" "${dictPath}/dict.dat"
            fi
            if [[ -d "${dictPath}/dict" ]]; then
                if [[ ! -e "${dictPath}/data/nvdaControllerClient.dll" ]]; then
                    cp "${cache}/nvda2speechd32.dll" "${dictPath}/data/nvdaControllerClient.dll"
                fi
                pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
            fi
            if [[ ! -d "${dictPath}/dict" ]] && [[ ! -r "${cache}/bk3-dict.dat" ]]; then
                find "${WINEPREFIX}/drive_c/nyanchangame/bk3" -type f -name 'nvdaControllerClient.dll' -exec rm -v "{}" \;
                "${0%/*}/speech/clipboard_translator.sh" play.exe bokurano-daibouken3 &
            fi
        fi
        if [[ "${game[0]}" == "bop-it-emulator" ]]; then
            "${0%/*}/speech/speak_window_title.sh" bop.exe &
        fi
        if [[ "${game[0]}" == "breu2-shadow-hunt" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
        if [[ "${game[0]}" == "road-to-rage" ]]; then
            "${0%/*}/speech/speak_window_title.sh" trtr.exe &
        fi
        if [[ "${game[0]}" == "sequence-storm" ]]; then
            "${0%/*}/speech/clipboard_reader.sh" SequenceStorm &
        fi
        if [[ "${game[0]}" == "shadow-line" ]]; then
            find "${WINEPREFIX}/drive_c/" -type f -name 'nvdaControllerClient.dll' -exec rm -v "{}" \;
            "${0%/*}/speech/clipboard_translator.sh" play_sr.exe shadow-line &
        fi
        if [[ "${game[0]}" == "sketchbook" ]]; then
            find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        fi
        if [[ "${game[0]}" == "audiodisc" ]]; then
            wine "$winePath\\$wineExec"
            exit 0
        fi
        if [[ "${game[0]}" == "audioquake" ]]; then
            wine "$winePath\\$wineExec"
            exit 0
        fi
        if [[ "${game[0]}" == "screaming-strike-2" ]]; then
            pushd "$(winepath "$winePath")"
            ${wine} "$wineExec"
            popd
            exit 0
        fi
        if [[ "${game[0]}" == "warsim" ]]; then
            pushd "$(winepath "${game[1]%\\*}")"
            wine "${game[1]##*\\}"
            popd
            exit 0
        fi
        if [[ "${game[0]}" == "interceptor" ]]; then
            pushd "$(winepath "$winePath")"
            wine "$wineExec"
            popd
            exit 0
        fi
        if [[ "${game[0]}" == "vip-mud" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
        if [[ -d "${WINEPREFIX}/drive_c/windows/syswow64" ]]; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
            # switch to wine64 for 64 bit prefix.
            [[ "${wine}" == "wine" ]] && export wine="wine64"
        fi
        if [[ "${game[0]}" == "the-great-toy-robbery" ]] ; then
            pgrep -u "$USER" nvda2speechd &> /dev/null || ${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd &
        fi
}

# Process game launcher flags
process_launcher-flags() {
    flags=("${game[@]:3}")
    for i in "${flags[@]}" ; do
        if [[ "${i}" =~ ^export\ [a-zA-Z_][a-zA-Z0-9_]*=\'?.*\'?$ ]]; then
            eval "${i}"
        fi
    done
}

create_game_array() {
    # Game array 0 bottle, 1 path, 2 title, 3+ flags
    for i in "${lines[@]}" ; do
        if [[ "${game}" =~ ^${i} ]]; then
            # This is weird. Why do I have to set game to i before making the array?
            game="$i"
            IFS='|' read -ra game <<< "$i"
            break
        fi
    done
}

# launch games that are installed
game_launcher() {
    # For use by update scripts that want to source functions in this file.
    [[ "$agmNoLaunch" == "true" ]] && return
    mapfile -t lines < <(sed -e '/^$/d' -e '/^ *#/d' "${configFile}" 2> /dev/null)
    if [[ ${#lines} -eq 0 ]]; then
        echo "Install some games first."
        exit 0
    fi
    if [[ $# -eq 0 ]]; then
        # Create the menu of installed games
        declare -a menuList
        for i in "${lines[@]}" ; do
            IFS='|' read -ra gameInfo <<< "$i"
            menuList+=("${gameInfo[0]}|${gameInfo[1]}" "${gameInfo[2]}")
        done
        menuList+=("Donate" "Donate")
        menuList+=("Become a Patron" "Become a Patron")
        game="$(dialog --backtitle "Audio Game Launcher" \
            --clear \
            --extra-button \
            --extra-label "Documentation" \
            --ok-label "Launch" \
            --no-tags \
            --menu "Please select a game to play" 0 0 0 "${menuList[@]}" --stdout)"
        local menuCode=$?
        if [[ $menuCode -eq 1 ]]; then
            exit 0
        elif [[ $menuCode -eq 3 ]]; then
        documentation "$game" "$(echo "$game" | cut -d '|' -f2)"
        fi
        create_game_array
    else
        create_game_array
        if [[ -z "$game" ]]; then
            echo "Game $1 not found."
            exit 1
        fi
    fi
    if [[ ${#game[@]} -gt 0 ]]; then
        if [[ "${game[0]}" == "Donate" ]]; then
            open_url "https://ko-fi.com/stormux"
            exit 0
        fi
        if [[ "${game[0]}" == "Become a Patron" ]]; then
            open_url "https://2mb.games/product/2mb-patron/"
            exit 0
        fi
        get_bottle "${game[0]}"
        # make sure wine is actually set to something
        export wine="${wine:-wine}"
        echo -n "launching "
        ${wine} --version
        # kill any previous existing wineservers for this prefix in case they didn't shut down properly.
        ${wine}server -k
        # launch the game
        if command -v qjoypad &> /dev/null ; then
            mkdir -p ~/.qjoypad3
            touch "${HOME}/.qjoypad3/${game[2]}.lyt"
            # A | separated list of games that should not start with qjoypad.
            noQjoypadGames="A Hero's Call"
            if [[ "${noqjoypad}" != "true" ]] && ! [[ "${game[2]}" =~ ${noQjoypadGames} ]]; then
                if pgrep qjoypad &> /dev/null ; then
                    qjoypad -T "${game[2]}" 2> /dev/null
                else
                    qjoypad -T "${game[2]}" 2> /dev/null &
                fi
            fi
        fi
        process_launcher-flags
        custom_launch_parameters
        ${wine:-wine} start /d "${game[1]%\\*}" "${game[1]##*\\}" /realtime
    fi
    exit 0
}


# main script

#functions

add_launcher() {
    local launchSettings="${game,,}"
    launchSettings="${launchSettings//[[:space:]]/-}|${1}|${game}"
    shift
    while [[ $# -gt 0 ]]; do
        launchSettings+="|$1"
        shift
    done
    if ! grep -F -q -x "${launchSettings}" "${configFile}" 2> /dev/null ; then
        echo "${launchSettings}" >> "${configFile}"
        sort -o "${configFile}" "${configFile}"
        # Remove .lnk files because they don't work.
        find ~/Desktop -type f -iname '*.lnk' -exec bash -c '
            for f ; do
                mimeType="$(file -b "$f")"
                mimeType="${mimeType%%,*}"
                if [[ "$mimeType" == "MS Windows shortcut" ]]; then
                    rm -v "$f"
                fi
            done' _ {} +
        if [[ "${noCache}" == "true" ]]; then
            rm -f "${cache}/${1##*\\}"
        fi
    fi
}

# Check for updates
update
# If display isn't set assume we are launching from console and an X environment is running using display :0
if [[ -z "$DISPLAY" ]]; then
    export DISPLAY=":0"
fi
# Settings file
cache="${XDG_CACHE_HOME:-$HOME/.cache}/audiogame-manager"
configFile="${XDG_CONFIG_HOME:-$HOME/.config}/storm-games/audiogame-manager/games.conf"
mkdir -p "${cache}"
mkdir -p "${configFile%/*}"
# Create the path for AGM's helper programs.
# Originally this was only winetricks, thus the name, and I'm too lazy to change it now.
winetricksPath="${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager"
mkdir -p "${winetricksPath}"
# Load any arguments from settings.conf file
if [[ -r "${configFile%/*}/settings.conf" ]]; then
    source "${configFile%/*}/settings.conf"
fi
# Update the cache for older versions of audiogame-manager
if [[ -d "${configFile%/*}/cache" ]]; then
    { mv -v "${configFile%/*}/cache/"* "${cache}"
    rmdir -v "${configFile%/*}/cache/"; } | dialog \
        --backtitle "Audiogame Manager" --progressbox "Updating cache, please wait..." -1 -1
fi
checkWinetricksUpdate="false"
# Turn off debug messages
export WINEDEBUG="${winedebug:--all}"
# Compatibility with box86
export BOX86_NOBANNER=1
# During installation, you can set winVer to the versions available.
# To set winetricks arguments, such as virtual desktop, set the winetricksSettings variable.
# Example: winetricksSettings="vd=1024x768"
# Files are cached unless -N no cache is set.
unset noCache
# Manual installation is not default, make sure it's unset
unset manualInstall
unset version
# ipfs gateway
export ipfsGateway="${ipfsGateway:-https://gateway.pinata.cloud}"
export nvdaControllerClientDll="${ipfsGateway}/ipfs/QmWu7YdSbKMk1Qm5DKvEA5hk1YuAK8wVkwhDf2CsmPkmF1?filename=nvdaControllerClient32.dll"


# The list of games available for installation.
# Use menu friendly names.
gameList=(
    "A Hero's Call"
    "Adrian's Doom"
    "Adventurers At C"
    "Alien Outback"
    "Angel Gift"
    "AudioDisc"
    "AudioQuake"
    "Balatro"
    "Battlefield 2D"
    "Battle of the Hunter"
    "Battle Zone"
    #"Beatstar Pro"
    "BG 2048"
    "BG 15 Puzzle"
    "BG Aces Up Solitaire"
    "BG Alchemy"
    "BG Battleship"
    "BG Boggle"
    "BG Boxes"
    "BG Brainiac"
    "BG Chess Challenge"
    "BG Code Breaker"
    "BG Cribbage"
    "BG Cribbage Solitaire"
    "BG Crossword Puzzle"
    "BG Draw Dominoes"
    "BG Fives Dominoes"
    "BG Elevens Solitaire"
    "BG Free Cell Solitaire"
    "BG Golf Solitaire"
    "BG Hangman"
    "BG Hearts"
    "BG Klondike Solitaire"
    "BG LAP"
    "BG Master Mind"
    "BG Mine Sweeper"
    "BG Nomination Whist"
    "BG Penguin Solitaire"
    "BG Poker Solitaire"
    "BG Pyramid Solitaire"
    "BG Scorpion Solitaire"
    "BG Simon"
    "BG Spider Solitaire"
    "BG Scrabble"
    "BG Sudoku"
    "BG Tablic Solitaire"
    "BG Tri-Peaks Solitaire"
    "BG Twenty 20 Cricket"
    "BG Uno"
    "BG Word Builder"
    "BG Word Candy"
    "BG Word Jumble"
    "BG Word Maze"
    "BG Word Solitaire"
    "BG Word Target"
    "BG Word Yahtzee"
    "BG Yahtzee"
    "Blind Drive"
    "Bloodshed"
    "Bokurano Daibouken"
    "Bokurano Daibouken 2"
    "Bokurano Daibouken 3"
    "Bombercats"
    "Bop It Emulator"
    "Bounce Bounce"
    #"Breed Memorial"
    "Breu2 Shadow Hunt"
    "Castaways"
    "Castaways 2"
    "Challenge of the Horse"
    "Change Reaction"
    "Chillingham"
    #"Chopper Challenge"
    "Christmas Chaos"
    "Christmas WhoopAss"
    "Clashes of the Sky"
    "Code Dungeon"
    "Coin Collector"
    "Conjury"
    #"Constant Battle"
    "Copter Mission"
    "Crazy Party"
    "Crazy Tennis"
    "Crime Hunter"
    "Danger City"
    "Danger on the Wheel"
    #"Dark Destroyer"
    "Daytona and the Book of Gold"
    "Death on the Road"
    "Deathmatch"
    "Dog Who Hates Toast"
    "Dragon Pong"
    "Dreamland"
    "Duck Hunt"
    "DynaMan"
    "Easter Quest"
    "Endless Runner"
    "Entombed"
    "ESP Pinball Classic"
    "ESP Pinball Extreme"
    "ESP Pinball Party Pack"
    #"Eurofly"
    "Executioner's Rage"
    "Extant"
    #"Fartman"
    "Finger Panic"
    "Fuck That Bird"
    "Galactic Strike"
    "GMA Tank Commander"
    "Grizzly Gulch"
    "Hammer of Glory"
    "haunted Party"
    #"Hearthstone"
    "Hunter"
    "Inquisitor's Heartbeat"
    "Insect Therapy"
    "Interceptor"
    "Judgement Day"
    "Kitchensinc Games"
    "Kringle Crash"
    "Laser Breakout"
    "Light Battles"
    "Light Cars"
    "Lockpick"
    "Lone Wolf"
    "Lost"
    "Lunimals"
    "Manamon"
    "Manamon 2"
    #"Marina Break"
    "Maze Craze"
    #"Minecraft"
    "Mist World"
    "Monkey Business"
    "MudSplat French"
    "MudSplat English"
    #"MudSplat Swedish"
    "Oh Shit"
    "Operation BlackSquare"
    "Pacman Talks"
    "Palace Punch Up"
    "Paladin of the Sky"
    "Park Boss"
    "Paw Prints"
    "Penta Path"
    "Perilous Hearts"
    "Pontes Kickups!"
    "Pigeon Panic"
    "Pipe 2 Blast Chamber"
    "Preludeamals"
    #"Psycho Strike"
    "Puzzle Divided"
    "Q9"
    "Rettou"
    "Revelation"
    "Rhythm Rage"
    "River Raiders"
    "Road to Rage"
    "Road to Rage Offline"
    "RS Games"
    "Run For Your Life"
    #"Sammy Center"
    "Sarah and the Castle of Witchcraft and Wizardry"
    "Scramble!"
    "Screaming Strike 2"
    "Scrolling Battles"
    "Sketchbook"
    "Sequence Storm"
    #"Shades of Doom 1.2"
    "Shades of Doom"
    "Shadow Line"
    "Shooter"
    "Silver Dollar"
    "Simple Fighter"
    "Skateboarder Pro"
    "Slender Lost Vision"
    "Sonic the Hedgehog"
    "Sonic Zoom"
    #"Space Defender"
    #"Star Treck Final Conflict"
    "Super Deekout"
    "Super Dogs Bone Hunt"
    "Super Egg Hunt"
    "Super Liam"
    "Super Mario Bros"
    "Survive the Wild"
    "Swamp"
    "Tactical Battle"
    "Technoshock"
    "Ten Pin Alley"
    "The Blind Swordsman"
    #"The Gate"
    "The Great Toy Robbery"
    "The Vale"
    "They'll Come from the Moon"
    "Thief"
    "Traders of Known Space"
    "Three D velocity"
    "Tomb Hunter"
    "Top Speed 2"
    "Top Speed 3"
    "Tarot Assistant"
    "Triple Triad"
    "Troopanum2"
    "Tube Sim"
    "Ultimate SounDoku"
    "Undead Assault"
    "Villains From Beyond"
    "VIP Mud"
    "Warsim"
    #"Wave of the Undead"
    #"Wolf Games Launcher"
    "World of War"
)

# Make sure the minimum of curl, sox, wine, and winetricks are installed
for i in curl sox wine winetricks ; do
    if ! command -v $i &> /dev/null ; then
        echo "Please install $i before continuing."
        exit 1
    fi
done
# Get latest news if available
check_news
# With no arguments, open the game launcher.
if [[ $# -eq 0 ]]; then
    game_launcher
fi

# Array of command line arguments
declare -A command=(
    [c]="Check your system for necessary components."
    [C]="Clear the cache. All game installers will be deleted."
    [D]="Create desktop shortcut. You can launch audiogame-manager from the desktop or applications menu."
    [d]="Debug mode, wine will be much more verbose when games are installed with this flag."
    [h]="This help screen."
    [i]="Install games."
    [I:]="Noninteractive game installation."
    [k]="Kill a running game that is stuck."
    [L]="Display license information."
    [l:]="Launch given game without interactive audiogame-manager menu specified by its wine bottle."
    [N]="No cache, delete the installer after it has been extracted."
    [n]="No RHVoice, do not install RHVoice when the game is installed."
    [P]="Print a list of packages required by audiogame-manager."
    [q]="No qjoypad. Does not launch qjoypad if it is detected."
    [R]="Redownload. Removes old versions of packages from cache before installing."
    [r]="Remove a game. This will delete all game data."
    [S:]="Speech rate. Requires -V voice name, the default is 7. Values 0-9 or A."
    [t]="Total games. Show how many games are currently available."
    [v:]="Select the voice to be installed, default is Bdl. Options are alan, bdl, clb, or slt."
    [V:]="Select the default voice for a bottle."
)

# Convert the keys of the associative array to a format usable by getopts
args="${!command[*]}"
args="${args//[[:space:]]/}"
while getopts "${args}" i ; do
    case "$i" in
        c) checklist;;
        C) clear_cache;;
        D) desktop_launcher;;
        d)
            unset WINEDEBUG
            game_installer
        ;;
        h) help;;
        i) game_installer;;
        I)
            game="${OPTARG}"
            break;;
        k) kill_game;;
        L) license;;
        l) game_launcher "${OPTARG}";;
        N) noCache="true";;
        n) norh="true";;
        P) checklist quiet;;
        q)
            noqjoypad="true"
            game_launcher;;
        R) redownload="true";;
        r) game_removal;;
        S) defaultRate="${OPTARG}";;
        t)
            dialog --backtitle "Audiogame Manager" \
                --infobox "There are currently ${#gameList[@]} games available." -1 -1
            exit 0
            ;;
        v) voiceName="${OPTARG}";;
        V) defaultVoice="${OPTARG}";;
    esac
done

# Install game based on the selection above.
case "${game}" in
    "A Hero's Call")
        download "https://blindgamers.com/downloads/a-heros-call-freeware.zip" "${nvdaControllerClientDll}"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        install_wine_bottle speechsdk corefonts
        # Dotnet is evil. That is all.
        LC_ALL=C DISPLAY="" winetricks -q dotnet462 xna40
        wineserver -k # Really!
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/a-heros-call-freeware.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\a-heros-call\A Hero's Call.exe"
    ;;
    "Adrian's Doom")
        download "https://agarchive.net/games/mt/adrian's%20doom.exe"
        install_wine_bottle speechsdk
        wine "${cache}/adrian's doom.exe" /silent
        add_launcher "c:\Program Files\Two Caring Citizens\Adrian's Doom!\adrian.exe"
    ;;
    "Adventurers At C")
        download "http://www.vgstorm.com/aac/aac.zip" "https://www.agarchive.net/games/vg/adventure%20at%20c%20stages.7z"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/aac" "${cache}/aac.zip"
        7z x -o"$WINEPREFIX/drive_c/Program Files/aac/stages" "${cache}/adventure at c stages.7z"
        add_launcher "c:\Program Files\aac\aac.exe"
    ;;
    "Alien Outback")
        download "http://download.dracoent.com/Windows/classic/AOSetup.exe"
        export bottle="draconis"
        export winVer="win7"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/AOSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/AOSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/AOSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\Alien Outback\ao.exe"
    ;;
    "Angel Gift")
download "https://erion.cf/files/ag_103.zip"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/Angel Gift" "$cache/ag_103.zip"
        add_launcher 'c:\Program Files\Angel Gift\ag.exe'
    ;;
    "AudioDisc")
        download "https://agarchive.net/games/other/audiodisc.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/audiodisc.zip"
        add_launcher "c:\Program Files\audiodisc\disco.exe"
    ;;
    "AudioQuake")
        download "https://github.com/matatk/agrip/releases/download/2020.0-beta1/AudioQuake+LDL_2020.0-beta1_Windows.zip" "https://stormgames.wolfe.casa/downloads/quake.zip"
        export winVer="win7"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/AudioQuake+LDL_2020.0-beta1_Windows.zip"
        unzip -d "$WINEPREFIX/drive_c/Program Files/AudioQuake/id1" "${cache}/quake.zip"
        add_launcher "c:\Program Files\AudioQuake\AudioQuake.exe"
        echo
        echo "After you launch the game, press tab then enter and it should begin speaking."
        alert
    ;;
    "Balatro")
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll" "https://stormgames.wolfe.casa/downloads/Tolk.dll"
        export WINEARCH=win64
        export winVer="win10"
        install_wine_bottle
        get_steam "2379780" "https://store.steampowered.com/app/2379780/Balatro/"
        find "$WINEPREFIX/drive_c/Program Files/Balatro" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "$cache/nvda2speechd64.dll" "{}" \;
        find "$WINEPREFIX/drive_c/Program Files/Balatro" -type f -name 'tolk.dll' -exec cp -v "$cache/Tolk.dll" "{}" \;
        add_launcher 'c:\Program Files\Balatro\Balatro.exe' 'export WINEDLLOVERRIDES=version=n,b'
    ;;
    "Battlefield 2D")
        get_installer "bf.zip" "https://tunmi13.itch.io/battlefield-2d"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win8"
        export bottle=tunmi13-64bit
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/bf.zip"
        find "${WINEPREFIX}/drive_c/Program Files/bf" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\bf\bf.exe"
    ;;
    "Battle of the Hunter")
        download "http://tunmi13.ddns.net/projects/bth.zip"
        export bottle="tunmi13"
        export winVer="win7"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/$game" "${cache}/bth.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\\${game}\bth.exe"
    ;;
    "Battle Zone")
        download "https://www.agarchive.net/games/gameMadnessInteractive/battle%20zone%2013.5%20setup.exe"
        export winVer="win7"
        install_wine_bottle speechsdk
        wine "${cache}/battle zone 13.5 setup.exe" /silent
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\Battle Zone\ss.exe"
    ;;
    "Beatstar Pro")
        download "https://oriolgomez.com/games/beat_windows.zip"
        # Sapi is broken on win64 for now, and this game doesn't support nvda it seems.
        export WINEARCH=win64
        export winVer="win7"
        install_wine_bottle mf
        unzip -d "$WINEPREFIX/drive_c/Program Files/Beatstar Pro" "${cache}/beat_windows.zip"
        add_launcher "c:\Program Files\Beatstar Pro\beatstar.exe"
    ;;
    "BG 2048")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmPNt3c78UBgEMrTH3eJ5eD2mCMdth6jwes1iDKGW24Uj5?filename=BG204832Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BG204832Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\BG2048B\BG2048.exe"
    ;;
    "BG 15 Puzzle")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmQiocMpMXoxejDftKKvmrR5xxpj1qcWcgkhBBwTcyijXg?filename=FPB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/FPB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\FifteenB\FifteenB.exe"
    ;;
    "BG Aces Up Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmTshtHBEV9dh7wFtaQpNUEYHZ3fBpuhSRZqc7k8HwmtPM?filename=ASB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/ASB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\AcesUpB\AcesUpB.exe"
    ;;
    "BG Alchemy")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/Qma76HXBhmKgMDeHH1XLePsaWzzzLsBS2HRL3c7MVwDokg?filename=BAC32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BAC32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\AlchemyB\AlchemyB.exe"
    ;;
    "BG Battleship")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/Qmaq9P9fxdLTEFMGg4mhHrRuUbPg6HgU3eYVJNqZUimHjo?filename=BGB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\BattleshipB\BGBattleship.exe"
    ;;
    "BG Boggle")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmQwWiJw9hDiPdfwDyL4XepeoD66ztVRi3HwbSjFFP4CNg?filename=BGB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\BoggleB\BoggleB.exe"
    ;;
    "BG Boxes")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmRn21tREXxXVSaDe9i54zEPzPSespjJAFBqu4DWocuagD?filename=BXB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BXB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\BoxesB\BoxesB.exe"
    ;;
    "BG Brainiac")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmWEdmTkQsjSqBgWUgnDajMf8QvQBbEF4Nxo6mhkXYzBtQ?filename=BRN32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BRN32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\BrainiacB\BrainiacB.exe"
    ;;
    "BG Chess Challenge")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmT2yBpU5Jqna18FxYtyWzi4xMGAY9PyJWStAskxCHqBDw?filename=BGC32Setup10d.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGC32Setup10d.exe" /silent
        add_launcher "c:\Program Files\Games\ChessB\BGChess.exe"
    ;;
    "BG Code Breaker")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmU486SssAdM7kPKwDyAKDLQs3Z92bG6wFjaLhzqDZCxAF?filename=BCB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BCB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\CodeBreakerB\BGCodeBreaker.exe"
    ;;
    "BG Cribbage")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmeFud3EPHy7wQe8UENgvh96HdAazEkwqA2AutCNkYvB3t?filename=BGC32Setup12e.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGC32Setup12e.exe" /silent
        add_launcher "c:\Program Files\Games\CribbageB\CribbageB.exe"
    ;;
    "BG Cribbage Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmbRUiknnNcibWD3NwK4DFZGNHWswBgsFidUzU1TFGJ5Ra?filename=BCS32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BCS32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\CribSolB\CribSolB.exe"
    ;;
    "BG Crossword Puzzle")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmZQGY9CeATEiWrSqsKBz4AN6jPgQuvbBZSpQoLiMjoDr2?filename=BGX32Setup10h.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGX32Setup10h.exe" /silent
        add_launcher "c:\Program Files\Games\CrosswordB\CrosswordB.exe"
    ;;
    "BG Draw Dominoes")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmZQGY9CeATEiWrSqsKBz4AN6jPgQuvbBZSpQoLiMjoDr2?filename=BDD32Setup.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BDD32Setup.exe" /silent
        add_launcher "c:\Program Files\Games\DrawDominoesB\DrawDominoesB.exe"
    ;;
    "BG Elevens Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmWWZByYL5CsDSi6gQLGcMyBL7zqD5hWXbPXJr3shRt5AQ?filename=ESB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/ESB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\ElevensB\ElevensB.exe"
    ;;
    "BG Fives Dominoes")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmSZt6dz7WQkNrFBmYq9n4WdYrrZyQAebTBPo46uHqCuNi?filename=BFD32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BFD32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\FivesDominoesB\FivesDominoesB.exe"
    ;;
    "BG Free Cell Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmVfQMMnqTD9Zm8Xwv7rGrUTdS9FXToq7Fv6wtQQVgbQGR?filename=BGF32Setup20.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGF32Setup20.exe" /silent
        add_launcher "c:\Program Files\Games\FreecellB\FreecellB.exe"
    ;;
    "BG Golf Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmfAp9EYou1pndLwYSdpYdUCHBv2DR94oFccQh1ii9JVLD?filename=GSB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/GSB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\GolfSolitaireB\GolfSolitaireB.exe"
    ;;
    "BG Hangman")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmXTPMmvw7JE2eLuPBLGSpkZqUn12TX7QEQZbX8qtp7GBx?filename=HMB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/HMB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\HangmanB\HangmanB.exe"
    ;;
    "BG Hearts")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmdU5ag1PRjvG28wNX7aNuJqZSVxaqEEKjgG6GoRoDT8k4?filename=BGH32Setup10b.exe"
        install_wine_bottle speechsdk
        wine "${cache}/${BGH32Setup10b.exe}" /silent
        add_launcher "c:\Program Files\Games\HeartsB\HeartsB.exe"
    ;;
    "BG Klondike Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmctBDvhQWwER94LvgauR7sMDxv9D1mS9cToV47orTCdzU?filename=BGK32Setup10b.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGK32Setup10b.exe" /silent
        add_launcher "c:\Program Files\Games\KlondikeB\KlondikeB.exe"
    ;;
    "BG LAP")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/Qma5WeCC9B2P5abRGX9nGYV8Zi9F8vfCCr4ehejP2bgmNm?filename=LAP32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/LAP32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\LAP\LAP.exe"
    ;;
    "BG Master Mind")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmP6cwMbirbBqAaG9JLfNRnD2dvJfh6nq74kfwxs5hN2RQ?filename=BMM32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BMM32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\MastermindB\BGMasterMind.exe"
    ;;
    "BG Mine Sweeper")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmRa54HroWjwxHYfKr6hdmP34sHW5G3ecuzcjMA5UBBVKa?filename=MSB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/MSB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\MinesweeperB\MinesweeperB.exe"
    ;;
    "BG Nomination Whist")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/Qmb7eGTMDgiaDC9muMW9n8bHoistGcNm1VgHc6sr7dRyHU?filename=BNW32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BNW32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\NomWhistB\NomWhistB.exe"
    ;;
    "BG Penguin Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmXKvQ6WNNSnDiSyYmvAhZXVdALnuhUGK7dSMQVkQNReJr?filename=BPS32Setup10c.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BPS32Setup10c.exe" /silent
        add_launcher "c:\Program Files\Games\PenguinB\PenguinB.exe"
    ;;
    "BG Poker Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmPLv74LiDgVGuiGhu9HuPhx3uoMm9QyCYk6jgeFUHjj3S?filename=BPS32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BPS32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\PokerSolB\PokerSolB.exe"
    ;;
    "BG Pyramid Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmaqXaBKD3xY2smhU2LcejXRTPnWZHqaTW9se8yRepLsHu?filename=PSB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/PSB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\PyramidB\PyramidB.exe"
    ;;
    "BG Scorpion Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmSxJs2MiLQ61Fgx6vCpSD7GmQziLiCEU3sZ3mgWc7RsJ8?filename=BSS32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BSS32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\ScorpionB\ScorpionB.exe"
    ;;
    "BG Simon")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmXtBCqB6VCFPaDYuLaFNP1BDtJSLCJdJZzgm61zMtrsQt?filename=BGS32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGS32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\SimonB\SimonB.exe"
    ;;
    "BG Spider Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmdWBaDnLVbKCJSpiqF675ew6nJ6KHUVXA5FEH3t3E7UAu?filename=SPB32Setup10b.exe"
        install_wine_bottle speechsdk
        wine "${cache}/SPB32Setup10b.exe" /silent
        add_launcher "c:\Program Files\Games\SpiderB\SpiderB.exe"
    ;;
    "BG Scrabble")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmVrwyPdJBnmc4wLW7oT2hexxXnXxs8bA7gfiqbnJsWJ16?filename=BGS32Setup20.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGS32Setup20.exe" /silent
        add_launcher "c:\Program Files\Games\ScrabbleB\ScrabbleB.exe"
    ;;
    "BG Sudoku")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmXCAHEVRGZBc8t45Jgn2vkxicwF9Aox6yz9XrQBdkv7WY?filename=SDB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/SDB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\SudokuB\SudokuB.exe"
    ;;
    "BG Tablic Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmYoiFQ6JuSXfZfZXT3SQDsYzMWLBu9rW9yivi1xiPjqZx?filename=SDB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/SDB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\TabSolB\BGTabSol.exe"
    ;;
    "BG Tri-Peaks Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmWJGvSR6iaQfMHM3XuGCkWxx285jkzSDdNSvvk3bSCH8S?filename=TPB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/TPB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\TriPeaksB\TriPeaksB.exe"
    ;;
    "BG Twenty 20 Cricket")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmWAk2TMHMvW6Kjc1sZBEPsxmCNHfY3nF1K723PCqaTa57?filename=TPB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/T20B32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\T20CricketB\CricketB.exe"
    ;;
    "BG Uno")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmVsfPkebSoTDwYSXF1n7y4P9eGJTgTcGXdrEjpcV8A3Dv?filename=BGU32Setup11a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGU32Setup11a.exe" /silent
        add_launcher "c:\Program Files\Games\UnoB\UnoB.exe"
    ;;
    "BG Word Builder")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmXtR49EZShyj15Tc9CXQpBYVmKNfZpp4515Epm16bviuH?filename=BWB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BWB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\WordBuilderB\WordBuilderB.exe"
    ;;
    "BG Word Candy")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmfTgfRzd4JMRqKSfDiz76iMorkaG19BqH1K7nRCCDwo4H?filename=WCB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/WCB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\WordCandyB\WordCandyB.exe"
    ;;
    "BG Word Jumble")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmYQWZZifzKJSuVRCC1SabwRmEDz95GdFvbzRvsBMmTt6e?filename=BWJ32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BWJ32Setup10.wineExec" /silent
        add_launcher "c:\Program Files\Games\WordJumbleB\WordJumbleB.exe"
    ;;
    "BG Word Maze")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmXPtj5PkVZjXpU3m6FAfm8MwVL6bQCvhEDoR385u6FGTL?filename=BWM32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BWM32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\WordMazeB\WordMazeB.exe"
    ;;
    "BG Word Solitaire")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmZp73ARDPqgnCz7zxfKeBHjNoHrgZSgg2NdQZR2sMyZGD?filename=WSB32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/WSB32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\WordSolitaireB\WordSolitaireB.exe"
    ;;
    "BG Word Target")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmWWZFXVHNtmNkH55oermWWtrMcQ8qVqL687B7kGFyeezq?filename=WTB32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/WTB32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\WordTargetB\WordTargetB.exe"
    ;;
    "BG Word Yahtzee")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmdicAVDegDktY3euVAC2PPn4YBGz96KedxYXNe4WDQaoq?filename=BWY32Setup10.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BWY32Setup10.exe" /silent
        add_launcher "c:\Program Files\Games\WordYahtzeeB\BGWordYahtzee.exe"
    ;;
    "BG Yahtzee")
        export bottle="bg"
        download "${ipfsGateway}/ipfs/QmZebvkKgFAADnb1cgW6Bz7wTYdUh82X61QdtW66KcvmpF?filename=BGY32Setup10a.exe"
        install_wine_bottle speechsdk
        wine "${cache}/BGY32Setup10a.exe" /silent
        add_launcher "c:\Program Files\Games\yahtzeeB\BGYahtzee.exe"
    ;;
    "Blind Drive")
        get_installer "Blind Drive 1.1.112.00i-win64.zip" "https://lofipeople.itch.io/blind-drive"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/blind-drive" "${cache}/Blind Drive 1.1.112.00i-win64.zip"
        # Weird work around to get keyboard working.
        winetricks -q usetakefocus=y
        winetricks -q usetakefocus=n
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\blind-drive\Blind Drive.exe"
    ;;
    "Bloodshed")
        download "${ipfsGateway}/ipfs/QmcTCTMep4zp5zTw8ZaXYpjtu9inNPn8bNzwhW6cX97egw?filename=bloodshed.exe"
        export winVer="win7"
        install_wine_bottle speechsdk
        cp "${cache}/bloodshed.exe" "$WINEPREFIX/drive_c/Program Files/"
        add_launcher "c:\Program Files\bloodshed.exe"
    ;;
    "Bokurano Daibouken")
        download "https://www.nyanchangames.com/softs/nn_setup.exe"
        export bottle="nyanchan"
        export winVer="win7"
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/nyanchangame/bk" "${cache}/nn_setup.exe"
        find "${WINEPREFIX}/drive_c/nyanchangame/bk/" -type f -name 'nvdaControllerClient.dll' -exec rm -v "{}" \;
        add_launcher "c:\nyanchangame\bk\play.exe"
    ;;
    "Bokurano Daibouken 2")
        download "https://www.nyanchangames.com/softs/nn2_setup.exe"
        export bottle="nyanchan"
        export winVer="win7"
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/nyanchangame/bk2" "${cache}/nn2_setup.exe"
        find "${WINEPREFIX}/drive_c/nyanchangame/bk2/" -type f -name 'nvdaControllerClient.dll' -exec rm -v "{}" \;
        add_launcher "c:\nyanchangame\bk2\play.exe"
    ;;
    "Bokurano Daibouken 3")
        if [[ ! -r "${cache}/bk3-dict.dat" ]]; then
            echo "http://www.nyanchangames.com/order/bk3translate.html" | xclip -selection clipboard 2> /dev/null
            dialog --backtitle "Audiogame manager" --msgbox "If you would like English translations, the file is available at http://www.nyanchangames.com/order/bk3translate.html. Save the dict.dat file to your Downloads or Desktop directory. For convenience the url has been copied to your clipboard. Press enter when you are ready to continue." -1 -1 --stdout 
        fi
        dictFile=""
        for i in "${HOME}/Downloads/dict.dat" "${HOME}/Desktop/dict.dat" ; do
            if [[ -r "$i" ]]; then
                dictFile="$i"
            fi
        done
        if [[ "${#dictFile}" -ge 3 ]] && [[ ! -r "${cache}/bk3-dict.dat" ]]; then
            dialog --backtitle "Audiogame manager" --yesno "Possible English translation file found at $dictFile. Would you like to use it for BK3?" -1 -1 --stdout && cp -v "$dictFile" "${cache}/bk3-dict.dat"
        fi
        download "https://www.nyanchangames.com/softs/nn3_setup.exe" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        export bottle="nyanchan"
        export winVer="win7"
        install_wine_bottle
        cp "${cache}/nvda2speechd" "${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd"
        chmod +x "${XDG_DATA_HOME:-$HOME/.local/share}/audiogame-manager/nvda2speechd"
        7z x -o"$WINEPREFIX/drive_c/nyanchangame/bk3" "${cache}/nn3_setup.exe"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        add_launcher "c:\nyanchangame\bk3\play.exe"
    ;;
    "Bop It Emulator")
        download "http://www.masonasons.me/softs/BopItEmulator3.1PasswordIsBopIt.7z"
        export winVer="win7"
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/Program Files/Bop It" "${cache}/BopItEmulator3.1PasswordIsBopIt.7z" -pBopIt
        add_launcher "c:\Program Files\Bop It\bop.exe"
    ;;
    "Bounce Bounce")
        get_installer "bounce_bounce.rar" "https://kavyapriya.itch.io/bounce-bounce"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export winVer="win7"
        export WINEARCH=win64
        install_wine_bottle
        unrar x "${cache}/bounce_bounce.rar" -op"$WINEPREFIX/drive_c/Program Files"
        find "${WINEPREFIX}/drive_c/Program Files/bounce_bounce" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\bounce_bounce\bounce.exe"
    ;;
    "Breu2 Shadow Hunt")
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        get_installer "breu2.zip" "https://breu.itch.io/shadowhunt"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/breu2.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\breu2\breu2.exe"
    ;;
    "Bombercats")
        download "http://oriolgomez.com/games/bombercats_en.zip"
        export bottle="oriol-gomez"
        export winVer="win7"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/bomvercats" "${cache}/bombercats_en.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\bomvercats\game.exe"
    ;;
    "Breed Memorial")
        download "https://hirotaka2014.sakura.ne.jp/mh0406/game/breed_memorial.zip" "${nvdaControllerClientDll}"
        export winVer="win7"
        install_wine_bottle cjkfonts speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/breed_memorial.zip"
        #find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\Breed memorial\Breed memorial\breed memorial.exe"
    ;;
    "Castaways")
        download "https://www.kaldobsky.com/audiogames/castaways.zip"
        export bottle="aprone"
        export winVer="win7"
        install_wine_bottle vb6run dx8vb speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/castaways" "${cache}/castaways.zip"
        wine "c:\Program Files\castaways\Checkup.exe" /verysilent
        add_launcher "c:\Program Files\castaways\Castaways.exe"
    ;;
    "Castaways 2")
        download "http://www.kaldobsky.com/audiogames/castaways2beta.zip"
        export bottle="aprone"
        export winVer="win7"
        install_wine_bottle vb6run dx8vb speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/castaways2" "${cache}/castaways2beta.zip"
        wine "c:\Program Files\castaways2\Checkup.exe" /verysilent
        add_launcher "c:\Program Files\castaways2\Castaways2.exe"
    ;;
    "Challenge of the Horse")
        download "http://tunmi13.ddns.net/projects/coth.zip"
        export bottle="tunmi13"
        export winVer="win7"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/$game" "${cache}/coth.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\\${game}\game.exe"
    ;;
    "Change Reaction")
        download "https://download.dracoent.com/Windows/ChangeReactionSetup.exe"
        export bottle="draconis"
        export winVer="win7"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/ChangeReactionSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/ChangeReactionSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/ChangeReactionSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\Change Reaction\ChangeReactionGui.exe"
    ;;
    "Chillingham")
        download "https://stormgames.wolfe.casa/downloads/chillingham.zip"
        install_wine_bottle vb6run mfc42
        unzip -d "$WINEPREFIX/drive_c/Program Files" "$cache/chillingham.zip"
        add_launcher "c:\Program Files\chillingham\Chillingham.exe"
    ;;
    "Chopper Challenge")
        # Freezes at menu
        download "https://www.agarchive.net/games/XSight/chopper%20challenge%20setup.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/chopper challenge setup.exe" /silent &
        xdotool sleep 5 key y 2> /dev/null
        wineserver -w
        echo "$USER|n/a" >> "$WINEPREFIX/drive_c/Program Files/x-sight interactive/chopper challenge/config.dat"
        add_launcher "c:\Program Files\x-sight interactive\chopper challenge\Chopper.exe"
    ;;
    "Coin Collector")
        export WINEARCH=win64
        export winVer="win7"
        download "https://www.dropbox.com/s/v55q7t9n84otmcd/coin%20collector.rar?dl=1" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        install_wine_bottle
        unrar x "$cache/coin collector.rar" -op"$WINEPREFIX/drive_c/Program Files"
        find "$WINEPREFIX/drive_c/Program Files/coin collector" -type f -name "nvdaControllerClient64.dll" -exec cp -v "$cache/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\coin collector\game.exe"
    ;;
    "Conjury")
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        get_steam 2684520 "https://store.steampowered.com/app/2684520/Conjury/"
        find "$WINEPREFIX/drive_c/Program Files/Conjury" -type f -path '*/x86_64/nvdaControllerClient.dll' -exec cp -v "$cache/nvda2speechd64.dll" "{}" \;
        find "$WINEPREFIX/drive_c/Program Files/Conjury" -type f -path '*/x86/nvdaControllerClient.dll' -exec cp -v "$cache/nvda2speechd32.dll" "{}" \;
        add_launcher 'c:\Program Files\Conjury\release\Conjury.exe'
    ;;
    "Constant Battle")
        export winVer="win7"
        download "https://renovagames.com/bc/BC-Setup.exe"
        install_wine_bottle cjkfonts speechsdk
        wine "${cache}/BC-Setup.exe" /silent
        #add_launcher "c:\Program Files\"
    ;;
    "Christmas Chaos")
        export WINEARCH=win64
        export winVer="win7"
        download "${ipfsGateway}/ipfs/QmYx11vsMDBgjPd1coZPGHxMXf2qtf4icqmB3Q9iUazyQv?filename=ChristmasChaos.zip" "https://stormgames.wolfe.casa/downloads/Tolk.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/ChristmasChaos.zip"
        find "${WINEPREFIX}" -type f -name 'Tolk.dll' -exec cp -v "${cache}/Tolk.dll" "{}" \;
        add_launcher "c:\Program Files\ChristmasChaos\ChristmasChaos.exe"
    ;;
    "Christmas WhoopAss")
        download "https://www.agarchive.net/games/draconis/christmas%20whoopass%20setup.exe"
        install_wine_bottle vb6run dx8vb
        wine "${cache}/christmas whoopass setup.exe" /sp- /silent
        add_launcher "c:\Program Files\Draconis Entertainment\Christmas Whoop Ass\wa.exe"
    ;;
    "Clashes of the Sky")
        get_installer "clashes_of_the_sky.zip" "https://tunmi13.itch.io/clashes-of-the-sky"
        export bottle="tunmi13"
        export winVer="win7"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/clashes_of_the_sky.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher 'c:\Program Files\clashes_of_the_sky\clash.exe'
    ;;
    "Code Dungeon")
        get_installer "codedungeon-win-64.zip" "https://stealcase.itch.io/codedungeon"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/code-dungeon" "${cache}/codedungeon-win-64.zip"
        # Weird work around to get keyboard working.
        winetricks -q usetakefocus=y
        winetricks -q usetakefocus=n
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\code-dungeon\Code Dungeon.exe"
    ;;
    "Copter Mission")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/copter_en.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/copter mission" "${cache}/copter_en.zip"
        add_launcher "c:\Program Files\copter mission\game.exe"
    ;;
    "Crazy Party")
        export WINEARCH=win64
        export winVer="win8"
        download "http://pragmapragma.free.fr/crazy-party/Crazy-Party-beta81.zip" "https://stormgames.wolfe.casa/downloads/Tolk.dll" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/Crazy-Party-beta81.zip"
        find "${WINEPREFIX}" -type f -name 'Tolk.dll' -exec cp -v "${cache}/Tolk.dll" "{}" \;
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\Crazy-Party-beta81\Crazy Party.exe"
    ;;
    "Crazy Tennis")
        download "https://www.agarchive.net/games/VIP/crazy%20tennis%20setup.exe"
        install_wine_bottle speechsdk
        wine "${cache}/crazy tennis setup.exe" /sp- /silent
        add_launcher "c:\Program Files\Crazytennis\crazytennis.exe"
    ;;
    "Crime Hunter")
        export WINEARCH=win64
        export winVer="win7"
        download "http://masonasons.me/softs/CH2.0Win.zip" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/crime-hunter" "${cache}/CH2.0Win.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\crime-hunter\ch.exe"
    ;;
    "Danger City")
        download "https://www.agarchive.net/games/xl/DangerCityBeta2.exe"
        install_wine_bottle vb6run dx8vb
        wine "$cache/DangerCityBeta2.exe" /silent &
        xdotool sleep 15 key --clearmodifiers --delay 200 Return 2> /dev/null
        xdotool sleep 5 key --clearmodifiers --delay 200 Return 2> /dev/null
        xdotool sleep 5key --clearmodifiers --delay 200 Return 2> /dev/null
        xdotool sleep 10 --clearmodifiers --delay 200 Return 2> /dev/null
        wineserver -w
        add_launcher 'c:\Program Files\Danger City\dc.exe'
    ;;
    "Danger on the Wheel")
        download "http://oriolgomez.com/games/wheel_en.zip"
        export bottle="oriol-gomez"
        export winVer="win7"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/danger on the wheel" "${cache}/wheel_en.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\danger on the wheel\game.exe"
    ;;
    "Dark Destroyer")
        export bottle=pbgames
        download "https://www.agarchive.net/games/pb/Dark-Destroyer-Setup.exe"
        install_wine_bottle speechsdk ie6
        wine "$cache/Dark-Destroyer-Setup.exe" /silent
        add_launcher 'c:\Pbgames\Dark_destroyer\darkdestroyer.exe'
    ;;
    "Daytona and the Book of Gold")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://kaldobsky.com/audiogames/Daytona.zip"
        install_wine_bottle vb6run dx8vb quartz corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/daytona" "${cache}/Daytona.zip"
        wine 'c:\Program Files\daytona\checkup.exe' /verysilent
        add_launcher "c:\Program Files\daytona\Daytona.exe"
;;
"Death on the Road")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/road_en.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/death on the road" "${cache}/road_en.zip"
        add_launcher "c:\Program Files\death on the road\game.exe"
    ;;
    "Deathmatch")
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.agarchive.net/games/realitySoftware/death%20match%20project%20alpha%20setup.exe"
        install_wine_bottle quartz speechsdk
        wine "${cache}/death match project alpha setup.exe" /silent
        add_launcher "c:\Program Files\reality software\death match project alpha\dm1.exe"
    ;;
    "Dog Who Hates Toast")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.kaldobsky.com/audiogames/dogwhohatestoast.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/dogwhohatestoast" "${cache}/dogwhohatestoast.zip"
        wine 'c:\Program Files\dogwhohatestoast\checkup.exe' /verysilent
        add_launcher "c:\Program Files\dogwhohatestoast\DogwhoHatesToast.exe"
        echo "Note: Dog who Hates Toast installed. Once you start the game, you must press tab until you hear sapi on to get speech." >&2
;;
    "Dragon Pong")
        export winVer="win7"
        install_wine_bottle
        download "https://www.iamtalon.me/games/dragonpong.zip"
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/dragonpong.zip"
        add_launcher "c:\Program Files\dragonpong\DragonPong.exe"
    ;;
    "Dreamland")
        download https://scwl-1251129685.cos.ap-shanghai.myqcloud.com/dreamland/Win/DreamLandSetup.exe
        install_wine_bottle speechsdk ole32
        $wine "${cache}/DreamLandSetup.exe" /silent
        ;;
    "Duck Hunt")
        export bottle="l-works"
        download "http://files.l-works.net/dhsetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/dhsetup.exe" /silent
        add_launcher "c:\Program Files\Lworks\Duck Hunt\duckhunt.exe"
    ;;
    "DynaMan")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/classic/DMSetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/DMSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/DMSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/DMSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\DynaMan\dm.exe"
    ;;
    "Easter Quest")
        download "https://agarchive.net/games/mt/easter%20quest%20setup.exe"
        install_wine_bottle
        wine "${cache}/easter quest setup.exe" /silent
        add_launcher "c:\Program Files\MTGames\Easter Quest\easter.exe"
    ;;
    "Endless Runner")
        download "http://www.masonasons.me/softs/EndlessRunner.7z"
        install_wine_bottle speechsdk
        7z x -y -o"$WINEPREFIX/drive_c/Program Files/Endless Runner" "${cache}/EndlessRunner.7z" -prunner
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\Endless Runner\runner.exe"
    ;;
    "Entombed")
        export version="6.18"
        download "http://blind-games.com/newentombed/EntombedSetup.exe" "https://download.microsoft.com/download/E/C/1/EC1B2340-67A0-4B87-85F0-74D987A27160/SSCERuntime-ENU.exe" "https://stormgames.wolfe.casa/downloads/Entombed.exe.config" "https://stormgames.wolfe.casa/downloads/mfplat.dll"
        install_wine "$version" "32"
        export winVer="win7"
        install_wine_bottle speechsdk msvcrt40 gdiplus ie7 wmp11 mf
        # Ok, more dotnet.
        LC_ALL=C DISPLAY="" winetricks -q dotnet40 xna40
        ${wine}server -k # Sigh.
        mkdir -p "${WINEPREFIX}/drive_c/temp"
        pushd "${WINEPREFIX}/drive_c/temp"
        7z x "${cache}/SSCERuntime-ENU.exe"
        ${wine} msiexec /i "${WINEPREFIX}/drive_c/temp/SSCERuntime_x86-ENU.msi" /q
        rm *
        popd
        pushd "${WINEPREFIX}/drive_c/Program Files/Microsoft SQL Server Compact Edition/v3.5"
        ${wine} regsvr32 sqlceoledb35.dll
        ${wine} regsvr32 sqlceca35.dll
        popd
        ${wine} "${cache}/EntombedSetup.exe" /silent
        pushd "${WINEPREFIX}/drive_c/Program Files/Entombed"
        cp ../Microsoft\ SQL\ Server\ Compact\ Edition/v3.5/Private/System.Data.SqlServerCe.Entity.dll ../Microsoft\ SQL\ Server\ Compact\ Edition/v3.5/Private/System.Data.SqlServerCe.dll .
        cp ../Microsoft\ SQL\ Server\ Compact\ Edition/v3.5/sql* .
        cp "${cache}/Entombed.exe.config" .
        popd
        if [ ! -f "${WINEPREFIX}/drive_c/windows/system32/mfplat.dll" ] ; then
            cp "${cache}/mfplat.dll" "${WINEPREFIX}/drive_c/windows/system32/"
        fi
        add_launcher "c:\Program Files\Entombed\Entombed.exe"
    ;;
    "ESP Pinball Classic")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/classic/PBCSetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/PBCSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/PBCSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/PBCSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\ESP Pinball Classic\pbc.exe"
    ;;
    "ESP Pinball Extreme")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/classic/PBXSetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/PBXSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/PBXSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/PBXSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\ESP Pinball Xtreme\pbx.exe"
    ;;
    "ESP Pinball Party Pack")
        export winVer="win7"
        export bottle="draconis"
        export WINEPREFIX="$HOME/.local/wine/draconis"
        # Only works in conjunction with esp pinball extreme.
        if ! [ -f "$HOME/.local/wine/$bottle/drive_c/Program Files/Draconis Entertainment/ESP Pinball Xtreme/pbx.exe" ] ; then
            echo "Error: You need to install ESP Pinball Extreme first to use this game. Please do so before continuing." >&2
            exit 1
        fi
        download "http://download.dracoent.com/Windows/classic/PP1Setup.exe"
        cp -v "${cache}/PP1Setup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/PP1Setup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/PP1Setup.exe"
        add_launcher "c:\Program Files\Draconis Entertainment\ESP Pinball Xtreme\pbx.exe"
    ;;
    "Eurofly")
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.stefankiss.sk/files/eurofly2/Launcher_1.2.zip" "https://www.stefankiss.sk/files/eurofly2/Eurofly_2_ful_setup.exe" 
        install_wine_bottle speechsdk comctl32
        wine "${cache}/Eurofly_2_ful_setup.exe" /silent
        unzip -o -d "$WINEPREFIX/drive_c/Eurofly" "${cache}/Launcher_1.2.zip"
        add_launcher "c:\Eurofly\launcher.exe"
        echo "Note: On first and sometimes later launch, Eurofly may take a very long time to download required files, please be patient..."
    ;;
    "Executioner's Rage")
        download "https://dl.tweesecake.app/rage/rage1.5.0.zip" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win10"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/rage1.5.0.zip"
        find "${WINEPREFIX}/drive_c/Program Files" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\rage\rage.exe"
        url="https://techcake.games/games/executioners-rage/"
        echo "Before you can login, you need to create an account at:"
        echo "$url"
        if echo "$url" | xclip -selection clipboard 2> /dev/null ; then
            message+="\n\nThe URL has been copied to the clipboard."
        fi
        alert
    ;;
    "Extant")
        download "https://agarchive.net/games/other/extant.zip"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/extant" "${cache}/extant.zip"
        add_launcher "c:\Program Files\extant\Extant.exe"
    ;;
    "Fartman")
        download "http://www.agarchive.net/games/bpc/fartman.exe"
        install_wine_bottle dx8vb vb6run
        wine "${cache}/fartman.exe" /silent
        #add_launcher "c:\Program Files\"
    ;;
    "Finger Panic")
        download "http://www.agarchive.net/games/bsc/FingerPanicSetup.exe"
        install_wine_bottle dx8vb vb6run
        wine "${cache}/FingerPanicSetup.exe" /sp- /silent
        add_launcher "c:\Program Files\Finger Panic 1.0\FingerPanic.exe"
    ;;
    "Fuck That Bird")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/bird_en.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/fuck that bird" "${cache}/bird_en.zip"
        add_launcher "c:\Program Files\fuck that bird\game.exe"
    ;;
    "Galactic Strike")
        get_installer "Galactic Strike 1.2.zip" "https://fusion-forged-games.itch.io/galactic-strike"
        export winVer="win10"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/Galactic Strike" "${cache}/Galactic Strike 1.2.zip"
        add_launcher "c:\Program Files\Galactic Strike\Galactic Strike.exe"
        echo "Use controls wasd to movi and navigate the menu."
        echo "Use m to fire and select items from the menu."
        alert
    ;;
    "GMA Tank Commander")
        download "http://www.gmagames.com/gtc120.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/gtc120.exe" /silent &
        xdotool sleep 10 key --clearmodifiers alt+y 2> /dev/null
        xdotool sleep 15 type --clearmodifiers --delay 100 "${USER^}" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool sleep 10 type --clearmodifiers --delay 100 "${HOSTNAME^}" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool sleep 10 type --clearmodifiers --delay 100 "na@na.na" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool sleep 10 type --clearmodifiers --delay 100 "uuuuuuu" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool key --clearmodifiers Return 2> /dev/null
        xdotool sleep 10 key --clearmodifiers Return 2> /dev/null
        wineserver -w
        echo "To accurately set your information, edit the file:"
        echo "${WINEPREFIX}/drive_c/Program\ Files/GMA\ Tank\ Commander/config.dat"
        echo "The default country is US. The fields are:"
        echo -e "\"Firstname Lastname\"\n\"Email address\"\n\"Country code\""
        add_launcher "c:\Program Files\GMA Tank Commander\gtc.exe"
    ;;
    "Grizzly Gulch")
        download "https://stormgames.wolfe.casa/downloads/grizzly-gulch.zip"
        install_wine_bottle vb6run mfc42
        unzip -d "$WINEPREFIX/drive_c/Program Files" "$cache/grizzly-gulch.zip"
        add_launcher "c:\Program Files\grizzly-gulch\Grizzly Gulch.exe"
        echo "If the combat sequences happen too slow for your tastes, while in the town square, you can use f12 to turn up the combat speed and f11 to lower it."
        alert
    ;;
    "Hammer of Glory")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/hammer_en.zip"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/hammer of glory" "${cache}/hammer_en.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\hammer of glory\game.exe"
    ;;
    "haunted Party")
        get_installer "hp.zip" "https://tunmi13.itch.io/haunted-party"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win8"
        export bottle=tunmi13-64bit
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/hp.zip"
        find "${WINEPREFIX}/drive_c/Program Files/hp" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\hp\hp.exe"
    ;;
    "Hearthstone")
        export winVer="win10"
        install_wine_bottle ie8 vcrun2015 speechsdk
        # This download url breaks the download function, so grab it manually.
        curl -L --output "${cache}/Hearthstone.exe" "https://us.battle.net/download/getInstaller?os=win&installer=Hearthstone-Setup.exe"
        curl -L --output "${cache}/Hearthstone_patch.zip" "https://github.com/HearthstoneAccess/HearthstoneAccess/releases/latest/download/patch.zip"
        wine "${cache}/Hearthstone.exe" /silent
        unzip -o "${cache}/Hearthstone_patch.zip" -d "${WINEPREFIX}/drive_c/Program Files/Hearthstone"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        #add_launcher "c:\Program Files\"
    ;;
    "Hunter")
        download "http://www.agarchive.net/games/bsc/HunterSetup.exe" "https://www.agarchive.net/games/bsc/BSC%20unlock%20code%20generator.7z"
        install_wine_bottle vb6run dx8vb
        # FIXME: Hacky, but it works. Install dotnet35 by itself so it actually doesn't hang.
        winetricks -q dotnet35sp1
        wineserver -k # Damn you, dotnet.
        wine "${cache}/HunterSetup.exe" /silent &
        xdotool sleep 10 key --clearmodifiers alt+y 2> /dev/null
        sleep 30 && wineserver -k # Sometimes the installer finishes but the wineserver has more processes that don't exit, so we can't depend on wineserver -w.
        mkdir -p "$WINEPREFIX/drive_c/Program Files/bsc-key-generator"
        7z x -o"$WINEPREFIX/drive_c/Program Files/bsc-key-generator" "${cache}/BSC unlock code generator.7z"
        echo "$USER"$'\n'"$(hostname)"$'\n'"none"$'\n'"US" > "$WINEPREFIX/drive_c/Program Files/Hunter/config.dat"
        unix2dos "$WINEPREFIX/drive_c/Program Files/Hunter/config.dat"
        if command -v xclip &> /dev/null && command -v xdotool &> /dev/null ; then
            wine "c:\Program Files\Hunter\HunterRegistration.exe" &
            xdotool sleep 10 key Return sleep 2 key Return sleep 2 key Alt+n
            sleep 1
            regcode="$(xclip -selection clipboard -o)"
            xdotool sleep 1 key Shift+Tab sleep 1 key Return
            # FIXME: Kind of hacky, but let's make sure it actually exitted.
            sleep 5
            wineserver -k
            wine "c:\Program Files\bsc-key-generator\BlindsoftwareUnlockCodeGenerator.exe" &
            xdotool sleep 10 key Return sleep 2 type h
            xdotool sleep 1 key Tab sleep 1 type $regcode
            xdotool sleep 1 key Tab sleep 1 key Return
            sleep 2
            regcode="$(xclip -selection clipboard -o)" # Might as well reuse the variable.
                        # FIXME: Kind of hacky, but let's make sure it actually exitted since I can't find a good way to exit this program.
            sleep 5
            wineserver -k
                     wine "c:\Program Files\Hunter\HunterRegistration.exe" &
            echo "$regcode" | xclip -selection clipboard
            xdotool sleep 10 key Return sleep 2 key Shift+Tab sleep 1 key Shift+Tab sleep 1 key Return
            # FIXME: Kind of hacky, but let's make sure it actually exitted.
            sleep 5
            wineserver -k
        else
            echo "Warning: You need xclip and xdotool for this installer to finish the registration process, however, you don't have them. Either remove the program, install the stated tools, and reinstall it, or go on your own. Caveat Emptor, go to jail, do not pass go, do not collect audiogame rewards and all that stuff."
        fi
        add_launcher "c:\Program Files\Hunter\HunterRun.exe"
    ;;
    "Inquisitor's Heartbeat")
        get_installer "Inquisitor_windows_ENG.zip" "https://www.audiogame.store/en/products/inquisitors-heartbeat--windows/ios-only"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/Inquisitor_windows_ENG.zip"
        # Weird work around to get keyboard working.
        winetricks -q usetakefocus=y
        winetricks -q usetakefocus=n
        add_launcher "c:\Program Files\Inquisitor_windows_ENG\Inquisitor's Heartbeat.exe"
    ;;
    "Insect Therapy")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/insect_en.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/insect therapy" "${cache}/insect_en.zip"
        add_launcher "c:\Program Files\insect therapy\game.exe"
    ;;
    "Interceptor")
        download "http://www.valiantgalaxy.com/interceptor/Interceptor0.0.2.0Installer.exe"
        install_wine_bottle
        wine "${cache}/Interceptor0.0.2.0Installer.exe" /silent
        add_launcher "c:\Program Files\VGA\interceptor\launch_interceptor.exe"
    ;;
    "Judgement Day")
        export bottle="l-works"
        download "http://files.l-works.net/judgmentdayfullsetup.exe"
        install_wine_bottle vb6run dx8vb quartz
        wine "${cache}/judgmentdayfullsetup.exe" /silent
cat << EOF > /tmp/judgementday.reg
Windows Registry Editor Version 5.00

[HKEY_CURRENT_USER\Software\VB and VBA Program Settings\judgmentday\config]
"name"="$USER"
EOF
        wine regedit /s /tmp/judgementday.reg
        rm /tmp/judgementday.reg
        add_launcher "c:\Program Files\Lworks\Judgment Day\judgmentday.exe"
    ;;
    "Kitchensinc Games")
        download "${ipfsGateway}/ipfs/QmdkLPig6Kp3AZTwKAhjrhhsEuvhFCFhm6SHLUQVeNNYCb?filename=kitchen.tar.xz"
        install_wine_bottle vb6run speechsdk dx8vb
        echo "Extracting files..."
        tar xf "${cache}/kitchen.tar.xz" -C "$WINEPREFIX/drive_c/Program Files/"
        add_launcher "c:\Program Files\Kitchen's Sink\gamemenu.exe"
    ;;
    "Kringle Crash")
        download "https://www.agarchive.net/games/blastbay/kringle%20crash%20setup.exe"
        install_wine_bottle
        wine "${cache}/kringle crash setup.exe" /silent
        add_launcher "c:\Program Files\Kringle Crash\kringlecrash.exe"
    ;;
    "Laser Breakout")
        # Currently only speaks in japanese, looking to see if we can find an english version.
        export bottle="nyanchan"
        export winVer="win7"
        download "https://www.agarchive.net/games/nyanchan/laser%20breakout.7z" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll" "https://stormgames.wolfe.casa/downloads/laser-breakout-options.dat"
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/nyanchangame/" "$cache/laser breakout.7z"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        cp -v "$cache/laser-breakout-options.dat" "$WINEPREFIX/drive_c/nyanchangame/laser breakout/options.dat"
        add_launcher "c:\nyanchangame\laser breakout\play.exe"
    ;;
    "Light Battles")
        export winVer="win7"
        download "https://prometheus-enterprises.com/games/CoL.exe" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/Program Files/Light Battles" "${cache}/CoL.exe"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient.dll" -exec cp -v "$cache/nvda2speechd32.dll" "{}" \;
        add_launcher "c:\Program Files\Light Battles\battles.exe"
    ;;
    "Light Cars")
        download "https://www.agarchive.net/games/lighttech/light%20cars%20setup.exe"
        install_wine_bottle dx8vb vb6run
        wine "${cache}/light cars setup.exe" &
        xdotool sleep 10 key --clearmodifiers alt+n sleep 1 key --clearmodifiers alt+a key --clearmodifiers space sleep 1 key --clearmodifiers alt+n sleep 1 key --clearmodifiers alt+n sleep 1 key --clearmodifiers alt+i sleep 30 key --clearmodifiers Tab sleep 1 key --clearmodifiers Return 2> /dev/null
        wineserver -w
        echo -e "${USER} ${HOST}\nna@na.na\nUS" > ~/.local/wine/light-cars/drive_c/Program\ Files/Lighttech\ Interactive/Light\ Cars/config.dat
        unix2dos ~/.local/wine/light-cars/drive_c/Program\ Files/Lighttech\ Interactive/Light\ Cars/config.dat
        add_launcher "c:\Program Files\Lighttech Interactive\Light Cars\lightCars.exe"
    ;;
    "Lockpick")
        export bottle="l-works"
        download "http://files.l-works.net/lockpicksetup.exe"
        install_wine_bottle vb6run dx8vb
        wine "${cache}/lockpicksetup.exe" /silent
        add_launcher "c:\Program Files\lWorks\Lockpick\lockpick.exe"
    ;;
    "Lone Wolf")
        download "http://www.gmagames.com/lw350.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/lw350.exe" /silent &
        xdotool sleep 10 key --clearmodifiers alt+y 2> /dev/null
        xdotool sleep 10 type --clearmodifiers --delay 100 "$USER" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool type --clearmodifiers --delay 100 "$HOSTNAME" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool type --clearmodifiers --delay 100 "na@na.na" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool type --clearmodifiers --delay 100 "uuuuuu" 2> /dev/null
        xdotool key --clearmodifiers Tab 2> /dev/null
        xdotool key --clearmodifiers Return 2> /dev/null
        wineserver -w
        add_launcher "c:\Program Files\Lone Wolf\lw.exe"
    ;;
        "Lost")
        export bottle=dan-z
        download "https://agarchive.net/games/danZ/lost.zip"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/lost" "$cache/lost.zip"
        find "$WINEPREFIX/drive_c/Program Files/lost" -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher 'c:\Program Files\lost\lost.exe'
    ;;
    "Lunimals")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://kaldobsky.com/audiogames/lunimals.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/lunimals" "${cache}/lunimals.zip"
        wine 'c:\Program Files\lunimals\checkup.exe' /verysilent
        add_launcher "c:\Program Files\lunimals\Lunimals.exe"
        echo "Note: Lunimals installed. Once you start the game, you must press tab until you hear sapi on to get speech." >&2
        alert
    ;;
    "Manamon")
        export winVer="win7"
        download "https://www.vgstorm.com/manamon/manamon_installer.exe"
        install_wine_bottle speechsdk
        wine "${cache}/manamon_installer.exe" /silent
        add_launcher "c:\Program Files\VGStorm.com\Manamon\rpg.exe"
    ;;
    "Manamon 2")
        export winVer="win7"
        download "http://www.vgstorm.com/manamon2/manamon2_installer.exe"
        install_wine_bottle speechsdk
        wine "${cache}/manamon2_installer.exe" /silent
        add_launcher "c:\Program Files\VGStorm.com\Manamon 2\rpg.exe"
    ;;
    "Marina Break")
        export bottle="nyanchan"
        export winVer="win7"
        download "https://www.nyanchangames.com/softs/MbSetupE.exe" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        install_wine_bottle speechsdk
        wine "${cache}/MbSetupE.exe" &
        xdotool sleep 10 key Return
        wineserver -w
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient.dll" -exec cp -v "$cache/nvda2speechd32.dll" "{}" \;
        add_launcher "c:\nyanchangame\MarinaBreak\marinabreak.exe"
    ;;
    "Maze Craze")
        export bottle=dan-z
        download "http://www.danielzingaro.com/maze_craze_setup.exe"
        install_wine_bottle vb6run dx8vb
        ${wine} "${cache}/maze_craze_setup.exe" &
        xdotool sleep 15 key --delay 100 y 2> /dev/null
        xdotool sleep 3 key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+a 2> /dev/null
        xdotool key --delay 250 space 2> /dev/null
        xdotool key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+i 2> /dev/null
        xdotool sleep 15 key --delay 250 alt+f 2> /dev/null
        xdotool sleep 3 key --delay 250 n 2> /dev/null
        ${wine}server -k
        add_launcher "c:\Program Files\Maze Craze 1.4\mazecraze.exe"
    ;;
    "Minecraft")
        export WINEARCH=win64
        export winVer="win7"
        get_installer "accessibility-plus-extended-1.18-curseforge-v1.9.0.jar" "https://www.curseforge.com/minecraft/mc-mods/accessibility-plus-extended/download"
        get_installer "Mambience-5.2.0+1.19.jar" "https://www.curseforge.com/minecraft/mc-mods/mambience/download"
        get_installer "easy-life-1.18-v1.5.1.jar" "https://www.curseforge.com/minecraft/mc-mods/easy-life/download"
        get_installer "numpad-camera-controls-1.18-v1.5.1.jar" "https://www.curseforge.com/minecraft/mc-mods/numpad-keypad-camera-controls/download"
        download "https://www.python.org/ftp/python/3.7.9/python-3.7.9-amd64.exe"
        install_wine_bottle
        wine "$cache/python-3.7.9-amd64.exe" /silent
        wine 'c:\windows\py.exe' -m pip install portablemc portablemc-fabric
        # Now just need to install latest fabric, portablemc start --dry fabric
        wine 'c:\users\'"$USER"'\AppData\Local\Programs\Python\Python37\Scripts\portablemc.exe' start --dry fabric:
        # Also need to copy all mods into right place.
        mkdir "$WINEPREFIX/drive_c/users/$USER/AppData/Roaming/.minecraft/mods"
        for x in "accessibility-plus-extended-1.18-curseforge-v1.9.0.jar" "Mambience-5.2.0+1.19.jar" "easy-life-1.18-v1.5.1.jar" "numpad-camera-controls-1.18-v1.5.1.jar" ; do
            cp "$cache/$x" "$WINEPREFIX/drive_c/users/$USER/AppData/Roaming/.minecraft/mods"
        done
        # And of course, add the launcher.
;;
    "Mist World")
        export winVer="win7"
        get_installer "Mist World_Setup.exe" "https://drive.google.com/file/d/12YeUqorkkMT46ZSR5pcfWxSY8DHOLxZ-/view?usp=share_link"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        install_wine_bottle ole32
        7z x -o"$WINEPREFIX/drive_c/Program Files/Mist World" "$cache/Mist World_Setup.exe"
        sed -i 's/1024m/768m/g' "$WINEPREFIX/drive_c/Program Files/Mist World/mw.exe.vmoptions"
        cp "$WINEPREFIX/drive_c/Program Files/Mist World/"{mw.exe.vmoptions,update.exe.vmoptions}
        find "$WINEPREFIX/drive_c/Program Files/Mist World" -iname "nvdaControllerClient32.dll" -exec cp "$cache/nvda2speechd32.dll" "{}" \;
        mkdir "$WINEPREFIX/drive_c/Program Files/Mist World/"{user,users}
        add_launcher 'c:\Program Files\Mist World\mw.exe'
        echo
        echo "If you do not have an account, There is a script in game-scripts to help."
        echo "Launch the game, press enter on create account, then drop into a console so the game window does not lose focus."
        echo "Change to the game-scripts directory and run"
        echo "./mist_world_account_creator.sh and follow the prompts."
        echo
        echo "To login, type your email address, press tab, and type your password."
        echo "If you want to enable automatic login, press tab two times followed by space, then tab and enter."
        echo "If you do not want to auto login, you can just press enter after typing your password."
        alert
    ;;
    "Monkey Business")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/classic/MBSetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/MBSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/MBSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/MBSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\Monkey Business\mb.exe"
    ;;
    "MudSplat English")
        a="a"
        f="f"
        i="i"
        n="n"
    ;&
    "MudSplat French")
        # Variables may be already set for English installation, so make sure not to overwrite them.
        a="${a:-j}"
        f="${f:-t}"
        i="${i:-i}"
        n="${n:-s}"
    ;&
    "MudSplat Swedish")
        # Variables may be already set for French installation, so make sure not to overwrite them.
        a="${a:-a}"
        f="${f:-f}"
        i="${i:-i}"
        n="${n:-n}"
        download "https://www.agarchive.net/games/other/Mudsplat-install.exe"
        install_wine_bottle
        wine "${cache}/Mudsplat-install.exe" &
        # Select the language.
        xdotool sleep 10 type --clearmodifiers ${game:9:1} 2> /dev/null
        xdotool sleep 1 key --clearmodifiers Return sleep 1 key alt+${n} sleep 1 key alt+${a} sleep 1 key alt+${n} sleep 1 key space sleep 1 key alt+${n} sleep 1 key alt+${n} sleep 1 key alt+${i} sleep 10 key space sleep 1 key alt+${f} 2> /dev/null
        wineserver -w
        mudsplatLauncher="$(find "$WINEPREFIX/drive_c/Program Files/TiM/MudSplat" -name 'mudsplat-*.exe')"
        mudsplatLauncher="${mudsplatLauncher##*/}"
        add_launcher "c:\Program Files\TiM\MudSplat\\${mudsplatLauncher}"
    ;;
    "Oh Shit")
        export winVer="win7"
        export norh="true" # Requires sapi even though uses nvda
        download "${ipfsGateway}/ipfs/QmQnAJJrt5uABFziQc7enXYrJ74J9GKQSMi8Ry8ebsxfPV?filename=OhShit.zip" "https://stormgames.wolfe.casa/downloads/nvda2speechd32.dll"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/OhShit.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        add_launcher "c:\Program Files\oh_shit\OhShit.exe"
    ;;
    "Operation BlackSquare")
        export winVer="win7"
        download "https://www.iamtalon.me/games/blacksquare.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/blacksquare.zip"
        add_launcher "c:\Program Files\blacksquare\OperationBlackSquare.exe"
    ;;
    "Pacman Talks")
        download "http://www.gmagames.com/pmt101.exe"
        install_wine_bottle
        wine "${cache}/pmt101.exe" /sp- /silent
        add_launcher "c:\Program Files\Pacman Talks\pmt.exe"
    ;;
    "Palace Punch Up")
        download "https://www.agarchive.net/games/blastbay/palace%20punch-up%20setup.exe"
        install_wine_bottle speechsdk
        wine "${cache}/palace punch-up setup.exe" /silent
        add_launcher "c:\Program Files\Palace Punch-up\palace.exe"
    ;;
    "Paladin of the Sky")
        export winVer="win7"
        download "http://www.vgstorm.com/cod/pots/paladin_installer.exe"
        install_wine_bottle speechsdk
        wine "${cache}/paladin_installer.exe" /silent
        add_launcher "c:\Program Files\VGStorm.com\Paladin of the Sky\game.exe"
    ;;
    "Park Boss")
        export winVer="win7"
        download "http://www.ndadamson.com/downloads/Park%20Boss%201.01%20setup.exe"
        install_wine_bottle speechsdk
        wine "${cache}/Park Boss 1.01 setup.exe" /silent &
        xdotool sleep 10 key --clearmodifiers Return sleep 1 key alt+n sleep 1 key alt+a sleep 1 key alt+i sleep 10 key alt+f 2> /dev/null
        wineserver -w
        add_launcher "c:\Program Files\NASoft\ParkBoss\pbMain.exe"
    ;;
    "Paw Prints")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.kaldobsky.com/audiogames/pawprints.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/pawprints" "${cache}/pawprints.zip"
        wine 'c:\Program Files\pawprints\checkup.exe' /verysilent
        add_launcher "c:\Program Files\pawprints\PawPrints.exe"
    ;;
    "Penta Path")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "http://www.kaldobsky.com/audiogames/pentapath.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/pentapath" "${cache}/pentapath.zip"
        wine 'c:\Program Files\pentapath\checkup.exe' /verysilent
        add_launcher "c:\Program Files\pentapath\PentaPath.exe"
    ;;
    "Perilous Hearts")
        download "https://www.agarchive.net/games/blastbay/perilous%20hearts%20concept%20demo.exe"
        install_wine_bottle speechsdk
        wine "${cache}/perilous hearts concept demo.exe" /silent
        add_launcher "c:\Program Files\Perilous Hearts Concept Demo\perilous_hearts.exe"
    ;;
    "Pigeon Panic")
        export bottle="l-works"
        download "http://agarchive.net/games/lworks/pigeon%20panic%20setup.exe"
        install_wine_bottle vb6run dx8vb
        wine "${cache}/pigeon panic setup.exe" /silent
        add_launcher "c:\Program Files\Lworks\Pigeon Panic\pigeonPanic.exe"
    ;;
    "Pipe 2 Blast Chamber")
        install_wine_bottle vb6run dx8vb
        # FIXME: Hacky, but it works. Install dotnet35 by itself so it actually doesn't hang.
        winetricks -q dotnet35sp1
        wineserver -k # Damn you, dotnet.
        download "http://www.agarchive.net/games/bsc/BlastChamberSetup.exe" "https://www.agarchive.net/games/bsc/BSC%20unlock%20code%20generator.7z"
        wine "${cache}/BlastChamberSetup.exe" /silent &
        xdotool sleep 10 key --clearmodifiers alt+y 2> /dev/null
        sleep 30 && wineserver -k # Sometimes the installer finishes but the wineserver has more processes that don't exit, so we can't depend on wineserver -w.
        mkdir -p "$WINEPREFIX/drive_c/Program Files/bsc-key-generator"
        7z x -o"$WINEPREFIX/drive_c/Program Files/bsc-key-generator" "${cache}/BSC unlock code generator.7z"
        echo "$USER"$'\n'"$(hostname)"$'\n'"none"$'\n'"US" > "$WINEPREFIX/drive_c/Program Files/Blast Chamber/config.dat"
        unix2dos "$WINEPREFIX/drive_c/Program Files/Blast Chamber/config.dat"
        if command -v xclip &> /dev/null && command -v xdotool &> /dev/null ; then
            wine "c:\Program Files\Blast Chamber\register.exe" &
            xdotool sleep 10 key Return sleep 2 key Return sleep 2 key Alt+n
            sleep 1
            regcode="$(xclip -selection clipboard -o)"
            xdotool sleep 1 key Shift+Tab sleep 1 key Return
            # FIXME: Kind of hacky, but let's make sure it actually exitted.
            sleep 5
            wineserver -k
            wine "c:\Program Files\bsc-key-generator\BlindsoftwareUnlockCodeGenerator.exe" &
            xdotool sleep 10 key Return sleep 2 type b
            xdotool sleep 1 key Tab sleep 1 type $regcode
            xdotool sleep 1 key Tab sleep 1 key Return
            sleep 2
            regcode="$(xclip -selection clipboard -o)" # Might as well reuse the variable.
                        # FIXME: Kind of hacky, but let's make sure it actually exitted since I can't find a good way to exit this program.
            sleep 5
            wineserver -k
                     wine "c:\Program Files\Blast Chamber\register.exe" &
            echo "$regcode" | xclip -selection clipboard
            xdotool sleep 10 key Return sleep 2 key Shift+Tab sleep 1 key Shift+Tab sleep 1 key Return
            # FIXME: Kind of hacky, but let's make sure it actually exitted.
            sleep 5
            wineserver -k
        else
            echo "Warning: You need xclip and xdotool for this installer to finish the registration process, however, you don't have them. Either remove the program, install the stated tools, and reinstall it, or go on your own. Caveat Emptor, go to jail, do not pass go, do not collect audiogame rewards and all that stuff."
        fi
        add_launcher "c:\Program Files\Blast Chamber\pipe.exe"
    ;;
    "Preludeamals")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.kaldobsky.com/audiogames/Preludeamals.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/preludeamals" "${cache}/Preludeamals.zip"
        wine 'c:\Program Files\preludeamals\checkup.exe' /verysilent
        add_launcher "c:\Program Files\preludeamals\Preludeamals.exe"
    ;;
    "Psycho Strike")
        download "http://www.vgstorm.com/psycho_strike_installer.exe"
        install_wine_bottle speechsdk
        wine "${cache}/psycho_strike_installer.exe" /silent
        add_launcher "c:\Program Files\VGStorm.com\Psycho Strike\strike.exe"
    ;;
    "Puzzle Divided")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.kaldobsky.com/audiogames/puzzledivided.zip"
        install_wine_bottle vb6run dx8vb quartz corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/puzzledivided" "${cache}/puzzledivided.zip"
        add_launcher "c:\Program Files\puzzledivided\PuzzleDivided.exe"
    ;;
    "Q9")
        download "http://www.blastbay.com/q9_english_installer.exe"
        install_wine_bottle
        wine "${cache}/q9_english_installer.exe" /silent
        add_launcher "c:\Program Files\Q9 Action Game\q9.exe"
    ;;
    "Rettou")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "http://www.kaldobsky.com/audiogames/rettou.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/rettou" "${cache}/rettou.zip"
        add_launcher "c:\Program Files\rettou\Rettou.exe"
    ;;
    "Revelation")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        download "https://www.kaldobsky.com/audiogames/revelation.zip"
        install_wine_bottle vb6run dx8vb quartz speechsdk corefonts
        unzip -d "$WINEPREFIX/drive_c/Program Files/revelation" "${cache}/revelation.zip"
        add_launcher "c:\Program Files\revelation\Revelation.exe"
    ;;
    "Rhythm Rage")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/rr_en.zip" "${nvdaControllerClientDll}"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/rhythm rage" "${cache}/rr_en.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\rhythm rage\game.exe"
    ;;
    "River Raiders")
        # No more choppy sound under water, woot!
        download "https://www.agarchive.net/games/XSight/River%20Raiders%201.3.5.exe"
        install_wine_bottle dsound directmusic
        wine "$cache/River Raiders 1.3.5.exe" &
        xdotool sleep 10 type y 2> /dev/null
        xdotool sleep 2 type y 2> /dev/null
        xdotool sleep 2 key --clearmodifiers alt+n sleep 2 key alt+n sleep 2 key alt+n sleep 2 key alt+i sleep 10 key alt+f 2> /dev/null
        wineserver -w
        add_launcher "c:\Program Files\River Raiders\raid.exe"
    ;;
    "Road to Rage")
        export WINEARCH=win64
        export winVer="win7"
        download "https://iamtalon.me/games/rtr_ultimate.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/rtr_ultimate.zip"
        add_launcher "c:\Program Files\rtr_Ultimate\trtr.exe"
    ;;
    "Road to Rage Offline")
        export winVer="win7"
        download "https://agarchive.net/games/talon/the%20road%20to%20rage%20offline.7z"
        install_wine_bottle speechsdk
        7z x -o"$WINEPREFIX/drive_c/Program Files" "${cache}/the road to rage offline.7z"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\RTR Offline\rtr.exe"
    ;;
    "RS Games")
        export version="7.0"
        download "http://rsgames.org/rsdownloads/rsgclient/rsgames-client-setup-2.01.exe" "${nvdaControllerClientDll}"
        install_wine "$version" "32"
        export winVer="win7"
        install_wine_bottle speechsdk
        ${wine} "${cache}/rsgames-client-setup-2.01.exe" /silent
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\RS Games Client\rsg.exe"
    ;;
    "Run For Your Life")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/rfyl_en.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/run for your life" "${cache}/rfyl_en.zip"
        add_launcher "c:\Program Files\run for your life\game.exe"
    ;;
    "Sammy Center")
        export winVer="win7"
        download "http://www.samtupy.com/games/SCSetup.exe" "${nvdaControllerClientDll}"
        install_wine_bottle speechsdk
        wine "${cache}/SCSetup.exe" /silent
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\Sam Tupy\SammyCenter\SammyCenter.exe"
    ;;
    "Sarah and the Castle of Witchcraft and Wizardry")
        download "http://www.pcs-games.net/Sarah10.exe" "http://www.pcs-games.net/Sarah-Patch4.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/Sarah10.exe" /sp- /verysilent
        wine "${cache}/Sarah-Patch4.exe" /sp- /verysilent
        add_launcher "c:\Program Files\Sarah and the Castle of Witchcraft and Wizardry 10\scw.exe"
    ;;
    "Scramble!")
        winetricksSettings="vd=1024x768"
        export winVer="win7"
        download "https://stevend.net/downloads/scramble_win32.zip" "${nvdaControllerClientDll}"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/scramble_win32.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\scramble_win32\scramble.exe"
    ;;
    "Screaming Strike 2")
        export bottle="nyanchan"
        export winVer="win7"
        download "https://www.nyanchangames.com/softs/screamingStrike2.exe" "${nvdaControllerClientDll}"
        install_wine_bottle fakejapanese speechsdk
        wine "${cache}/screamingStrike2.exe" &
        xdotool sleep 10 key Return
        wineserver -w
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\nyanchangame\Screaming Strike 2\play.exe"
    ;;
    "Scrolling Battles")
        export WINEARCH=win64
        export winVer="win7"
        get_installer "sbrw-win.zip" "https://masonasons.itch.io/sbrw"
        download "https://stormgames.wolfe.casa/downloads/Tolk.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/sbrw" "${cache}/sbrw-win.zip"
        find "${WINEPREFIX}" -type f -name 'Tolk.dll' -exec cp -v "${cache}/Tolk.dll" "{}" \;
        add_launcher "c:\Program Files\sbrw\sb.exe"
    ;;
    "Sketchbook")
        export winVer="win7"
        download "http://sbyw.games/SBYW/SBYW.zip" "http://sbyw.games/SBYW/sounds.zip" "${nvdaControllerClientDll}"
        install_wine_bottle speechsdk
        mv -v "${cache}/sounds.zip" "${cache}/SBYW-sounds.zip"
        unzip -d "$WINEPREFIX/drive_c/Program Files/sketchbook" "${cache}/SBYW.zip"
        unzip -d "$WINEPREFIX/drive_c/Program Files/sketchbook" "${cache}/SBYW-sounds.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\sketchbook\SBYW.exe"
    ;;
    "Sequence Storm")
        get_installer "sequence-storm-win64.zip" "https://special-magic-games-llc.itch.io/sequence-storm"
        export WINEARCH=win64
        export winVer="win10"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/sequence-storm" "${cache}/sequence-storm-win64.zip"
        write_sequence_storm_reader
        curl -L --output "$WINEPREFIX/drive_c/Program Files/sequence-storm/settings.json" "https://stormgames.wolfe.casa/downloads/sequencestorm-settings.json"
        add_launcher "c:\Program Files\sequence-storm\SequenceStorm.exe"
    ;;
    "Shades of Doom 1.2")
        download "http://gmagames.com/sod1208.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/sod1208.exe" /sp- /verysilent
        add_launcher "c:\Program Files\Shades of Doom 1.2\sod.exe"
    ;;
    "Shades of Doom")
        export winVer="win7"
        download "http://www.gmagames.com/sod20022.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/sod20022.exe" /silent
        add_launcher "c:\Program Files\Shades of Doom 2.0\sod.exe"
    ;;
    "Shadow Line")
        export version="7.7"
        download "https://www.mm-galabo.com/sr/Download_files_srfv/shadowrine_fullvoice3.171.exe" "https://raw.githubusercontent.com/LordLuceus/sr-english-localization/master/language_en.dat"
        install_wine "$version" "32"
        export winVer="win8"
        install_wine_bottle
        $wine "${cache}/shadowrine_fullvoice3.171.exe" /sp- &
        xdotool sleep 30 key --clearmodifiers --delay=500 Return
        xdotool key --clearmodifiers --delay=500 Return
        xdotool key --clearmodifiers --delay=500 Return
        xdotool key --clearmodifiers --delay=500 Return
        echo "Running installer, this will take approximately 3 minutes..."
        xdotool sleep 180 key --clearmodifiers --delay=500 Down
        xdotool key --clearmodifiers --delay=500 space
        xdotool key --clearmodifiers --delay=500 alt+f
        ${wine}server -w
        mv -v "${cache}/language_en.dat" "${WINEPREFIX}/drive_c/Program Files/GalaxyLaboratory/ShadowRine_FullVoice/SystemData/language_en.dat"
        add_launcher "c:\Program Files\GalaxyLaboratory\ShadowRine_FullVoice\play_sr.exe"
        echo "Please set the language to English when the game opens."
        echo "Go to options and press enter."
        echo "Press down arrow 5 times and press enter."
        echo "Press down arrow 1 time and press enter."
        echo "Press up arrow 2 times and press enter."
        echo "If everything worked as expected you should be back on the game menu and speech should work."
        alert
    ;;
    "Silver Dollar")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/SilverDollarSetup.exe"
        install_wine_bottle speechsdk
        cp -v "${cache}/SilverDollarSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/SilverDollarSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/SilverDollarSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet40 by itself so it actually doesn't hang.
        DISPLAY="" winetricks -q dotnet40
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\Silver Dollar\SilverDollarGui.exe"
    ;;
    "Simple Fighter")
        export WINEARCH=win64
        export winVer="win7"
        get_installer "simple fighter.exe" "https://tsatria03.itch.io/simple-fighter"
        if [[ ! -r "${cache}/simple-fighter-sounds.7z" ]] && [[ ! -r "${cache}/sounds.7z" ]]; then
            get_installer "sounds.7z" "https://tsatria03.itch.io/simple-fighter"
            mv "${cache}/sounds.7z" "${cache}/simple-fighter-sounds.7z"
        fi
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/Program Files/simple fighter" "${cache}/simple fighter.exe"
        7z x -o"$WINEPREFIX/drive_c/Program Files/simple fighter/sounds" "${cache}/simple-fighter-sounds.7z"
        find "${WINEPREFIX}/drive_c/Program Files/bf" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\simple fighter\game.exe"
    ;;
   "Slender Lost Vision")
        export winVer="win7"
        download "https://www.iamtalon.me/games/slender.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/slender.zip"
        add_launcher "c:\Program Files\slender\slender.exe"
   ;;
   "Shooter")
        export WINEARCH=win64
        export winVer="win7"
        get_installer "shooter-win.zip" "https://brynify.itch.io/shooter"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/shooter" "${cache}/shooter-win.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\shooter\shooter.exe"
   ;;
    "Skateboarder Pro")
        download "https://tunmi13.com/projects/sb_pro_rw.zip" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win8"
        export bottle=tunmi13-64bit
        install_wine_bottle sapi
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/sb_pro_rw.zip"
        find "${WINEPREFIX}/drive_c/Program Files/sb_pro_rw" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\sb_pro_rw\sb_pro.exe"
    ;;
    "Sonic the Hedgehog")
        export winVer="win7"
        download "https://www.agarchive.net/games/jeqoconGames/sonic%20the%20hedgehog.7z"
        install_wine_bottle speechsdk
        7z x -o"$WINEPREFIX/drive_c/Program Files/Sonic the Hedgehog" "${cache}/sonic the hedgehog.7z"
        add_launcher "c:\Program Files\Sonic the Hedgehog\sth.exe"
    ;;
   "Sonic Zoom")
        export winVer="win7"
        download "http://wwwx.cs.unc.edu/Research/assist/et/projects/SonicZoom/soniczoom11.zip"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/soniczoom11.zip"
        add_launcher "c:\Program Files\Sonic Zoom\SonicZoom.exe"
   ;;
   "Space Defender")
        export WINEARCH="win64"
        export winVer="win7"
        download "http://tunmi13.ddns.net/projects/space_defender.zip" "https://stormgames.wolfe.casa/downloads/nvdaControllerClient64.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/" "${cache}/space_defender.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvdaControllerClient64.dll" "{}" \;
        add_launcher "c:\Program Files\space_defender\sdefender.exe"
   ;;
    "Star Treck Final Conflict")
        download "https://agarchive.net/games/USA/star%20trek%20final%20conflict%20Setup.exe"
        install_wine_bottle speechsdk
        wine "${cache}/star trek final conflict Setup.exe" /silent
        add_launcher "c:\Program Files\USA Games\Final Conflict\stfc.exe"
    ;;
    "Super Deekout")
        export bottle=dan-z
        download "http://www.danielzingaro.com/superdeekout_setup.exe" "http://www.danielzingaro.com/sd_full.exe"
        install_wine_bottle vb6run dx8vb
        ${wine} "${cache}/superdeekout_setup.exe" &
        xdotool sleep 15 key --delay 100 y 2> /dev/null
        xdotool sleep 3 key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+a 2> /dev/null
        xdotool key --delay 250 space 2> /dev/null
        xdotool key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+n 2> /dev/null
        xdotool key --delay 250 alt+i 2> /dev/null
        xdotool sleep 15 key --delay 250 alt+f 2> /dev/null
        xdotool sleep 3 key --delay 250 n 2> /dev/null
        ${wine}server -k
        7z x -y -o"$WINEPREFIX/drive_c/Program Files/Super Deekout" "${cache}/sd_full.exe"
        echo "Super Deekout needs some information before it will run:"
        alert
        read -erp "Please enter your first name: " fname
        read -erp "Please enter your last name: " lname
        read -erp "Please enter your email address: " email
        ${wine} "c:\Program Files\Super Deekout\config.exe" &
        xdotool sleep 5 key --delay 100 Return 2> /dev/null
        xdotool sleep 3 type --clearmodifiers --delay 100 "${fname}"
        xdotool key --clearmodifiers --delay 100 Tab 2> /dev/null
        xdotool sleep 1 type --clearmodifiers --delay 100 "${lname}"
        xdotool key --clearmodifiers --delay 100 Tab 2> /dev/null
        xdotool sleep 1 type --clearmodifiers --delay 100 "${email}"
        xdotool key --clearmodifiers --delay 100 Tab 2> /dev/null
        xdotool sleep 15 key --delay 100 u 2> /dev/null
        xdotool key --clearmodifiers --delay 100 Tab 2> /dev/null
        xdotool sleep 1 key --delay 250 Return 2> /dev/null
        ${wine}server -k
        add_launcher "c:\Program Files\Super Deekout\super.exe"
    ;;
    "Super Dogs Bone Hunt")
        download "http://www.pcs-games.net/SBH11.exe"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/SBH11.exe" /sp- /silent
        add_launcher "c:\Program Files\SuperDog's Bone Hunt\sbh.exe"
    ;;
    "Super Egg Hunt")
        export bottle="l-works"
        download "http://files.l-works.net/superegghuntsetup.exe"
        install_wine_bottle
        wine "${cache}/superegghuntsetup.exe" /silent
        add_launcher "c:\Program Files\Lworks\super egg hunt\superegghunt.exe"
    ;;
    "Super Liam")
        export bottle="l-works"
        download "http://files.l-works.net/superliamsetup.exe"
        install_wine_bottle vb6run dx8vb
        wine "${cache}/superliamsetup.exe" /silent
        add_launcher "c:\Program Files\lWorks\Super Liam\sl.exe"
    ;;
    "Super Mario Bros")
        export winVer="win7"
        download "https://www.agarchive.net/games/jeqoconGames/super%20mario%20bros.7z"
        install_wine_bottle speechsdk
        7z x -o"$WINEPREFIX/drive_c/Program Files/Super Mario Bros" "${cache}/super mario bros.7z"
        add_launcher "c:\Program Files\Super Mario Bros\Mario.exe"
    ;;
    "Survive the Wild")
        export WINEARCH=win64
        export winVer="win8"
        download "https://stw.samtupy.com/files/stw.zip" "https://stormgames.wolfe.casa/downloads/Tolk.dll"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/Survive the Wild" "${cache}/stw.zip"
        find "${WINEPREFIX}" -type f -name 'Tolk.dll' -exec cp -v "${cache}/Tolk.dll" "{}" \;
        add_launcher "c:\Program Files\Survive the Wild\stw.exe"
        echo "When the game first launches, press enter to select English."
        echo "Press alt+a to accept the license agreement."
        echo "Press alt+a to accept the game rules."
        echo "Press enter repeatedly until you hear the game logo."
        alert
    ;;
    "Swamp")
        export bottle="aprone"
        export winVer="win7"
        export winetricksSettings="vd=1024x768"
        dialog --backtitle "Audiogame manager" --yesno "If you do not have a full 32 bit gstreamer installation, the Swamp music can cause stuttering and crashes. Would you like to remove the music directory after installation?" -1 -1 --stdout
        deleteMusic=$?
        download "https://www.kaldobsky.com/audiogames/Swamp.zip"
        install_wine_bottle dx8vb quartz corefonts vb6run speechsdk ole32
        unzip -d "$WINEPREFIX/drive_c/Program Files/swamp" "${cache}/Swamp.zip"
        # make sure the latest version is installed.
        if curl -L --output "${cache}/SwampPatch.zip" "https://www.kaldobsky.com/audiogames/SwampPatch.zip" ; then
            unzip -o -d "$WINEPREFIX/drive_c/Program Files/swamp" "${cache}/SwampPatch.zip"
        fi
        $wine 'c:\Program Files\swamp\checkup.exe' /verysilent
        #$wine cmd.exe /c 'cd /d c:\Program Files\swamp && Windows32bit.bat'
        # Delete music if requested.
        if [[ $deleteMusic -eq 0 ]]; then
            rm -frv "$WINEPREFIX/drive_c/Program Files/swamp/sounds/Music/"
        fi
        rm -fv "$WINEPREFIX/drive_c/Program Files/swamp/maps/sub2"
        add_launcher "c:\Program Files\swamp\Swamp.exe"
    ;;
    "Tactical Battle")
        download "https://blindgamers.com/downloads/Tactical%20Battle%20Dev.zip"
        install_wine_bottle speechsdk
        LC_ALL=C DISPLAY="" winetricks -q dotnet462
        ${wine}server -k
        unzip -d "$WINEPREFIX/drive_c/Program Files/Tactical Battle" "${cache}/Tactical Battle Dev.zip"
        add_launcher "c:\Program Files\Tactical Battle\Tactical Battle.exe"
    ;;
    "Technoshock")
        download "http://tiflocomp.ru/download/games/technoshock_140b_en.zip" "http://tiflocomp.ru/download/games/technoshock140b_en_update.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/technoshock_140b_en.zip"
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/technoshock_140b_en_update.zip"
        wine "$WINEPREFIX/drive_c/Program Files/setup_eng.exe" /silent
        wineserver -w
        wine "$WINEPREFIX/drive_c/Program Files/setup_eng_update_pack.exe" /silent
        add_launcher "c:\Program Files\Tiflocomp Games\Technoshock\ts.exe"
    ;;
    "Ten Pin Alley")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/classic/TPAXPSetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/TPAXPSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/TPAXPSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/TPAXPSetup.exe"
        #winetricks -q msdxmocx # I think having this installed first breaks things.
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\Ten Pin Alley\tpa.exe"
    ;;
    "The Blind Swordsman")
        download "https://www.agarchive.net/games/other/the%20blind%20swordsmanPC.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/the blind swordsmanPC.zip"
        add_launcher "c:\Program Files\TheBlindSwordsman.exe"
    ;;
    "The Gate")
        export winVer="win7"
        download "http://www.vgstorm.com/the_gate_installer.exe"
        install_wine_bottle
        wine "${cache}/the_gate_installer.exe" /silent
        add_launcher "c:\Program Files\VGStorm.com\The Gate\gate.exe"
    ;;
    "The Great Toy Robbery")
        export bottle="l-works"
        export winVer="win7"
        download "http://files.l-works.net/tgtrsetup_2.04.exe" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        install_wine_bottle dsound directmusic
        wine "${cache}/tgtrsetup_2.04.exe" /silent
        find "${WINEPREFIX}/drive_c/Program Files/Lworks/The Great Toy Robbery" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        add_launcher "c:\Program Files\Lworks\The Great Toy Robbery\tgtr.exe"
    ;;
    "The Vale")
        get_installer "the-vale-win.zip" "https://falling-squirrel.itch.io/the-vale"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/the-vale" "${cache}/the-vale-win.zip"
        # Weird work around to get keyboard working.
        winetricks -q usetakefocus=y
        winetricks -q usetakefocus=n
        add_launcher "c:\Program Files\the-vale\TheVale.exe"
    ;;
    "They'll Come from the Moon")
        get_installer "theyll-come-from-the-moon-windows.zip" "https://soundrascal.itch.io/theyll-come-from-the-moon"
        download "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win8"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/tcftm" "${cache}/theyll-come-from-the-moon-windows.zip"
        # Weird work around to get keyboard working.
        winetricks -q usetakefocus=y
        winetricks -q usetakefocus=n
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\tcftm\They'll Come from the Moon!.exe"
        echo "When the game launches, press the enter key to load the game which will begin to speak."
        alert
    ;;
    "Thief")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "http://oriolgomez.com/games/thief_en.zip"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/thief" "${cache}/thief_en.zip"
        add_launcher "c:\Program Files\thief\game.exe"
    ;;
    "Top Speed 2")
        download "http://users.pandora.be/playinginthedark/downloads/Tspeed_2.00.exe"
        install_wine_bottle
        wine "${cache}/Tspeed_2.00.exe" /silent
        add_launcher "c:\Program Files\Playing in the dark\Top Speed 2\TopSpeed.exe"
    ;;
    "Tomb Hunter")
        download "http://masonasons.me/softs/th_freeware_password_is_tombhunter.7z"
        install_wine_bottle speechsdk
        7z x -o"$WINEPREFIX/drive_c/Program Files/Tomb Hunter" "${cache}/th_freeware_password_is_tombhunter.7z" -ptombhunter
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\Tomb Hunter\th.exe"
    ;;
    "Top Speed 3")
        download "https://github.com/PlayingintheDark/TopSpeed/releases/download/h/Tspeed_3.0.3.exe"
        install_wine_bottle directplay
        wine "${cache}/Tspeed_3.0.3.exe" /silent
        add_launcher "c:\Program Files\Playing in the dark\Top Speed 3\TopSpeed.exe"
    ;;
    "Traders of Known Space")
        download "http://www.valiantgalaxy.com/tks/tradersOfKnownSpace0.0.0.5Installer.exe"
        install_wine_bottle
        wine "${cache}/tradersOfKnownSpace0.0.0.5Installer.exe" /silent
        add_launcher "c:\Program Files\VGA\TKS\tradersOfKnownSpace.exe"
    ;;
    "Three D velocity")
        export winVer="win10"
export WINEARCH=win64
export norh=false # Must install a voice, and rhvoice works easily with 64 bit.
        download "https://github.com/munawarb/Three-D-Velocity-Binaries/archive/master.zip" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        install_wine_bottle  sapi vcrun2008 gdiplus xact dotnet48 xna40
        # Dotnet is evil. That is all.
#        LC_ALL=C winetricks -q dotnet48 
#        wineserver -k # Ha ha ha.
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/master.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\Three-D-Velocity-Binaries-master\tdv.exe"
    ;;
    "Tarot Assistant")
        export bottle="aprone"
        export winVer="win7"
        download "https://www.kaldobsky.com/audiogames/tarot.zip"
        install_wine_bottle vb6run dx8vb speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/Tarot Assistant" "${cache}/tarot.zip"
        wine "c:\Program Files\Tarot Assistant\Checkup.exe" /verysilent
        add_launcher "c:\Program Files\Tarot Assistant\TarotAssistant.exe"
    ;;
    "Triple Triad")
        export bottle="aprone"
        export winVer="win7"
        download "https://www.kaldobsky.com/audiogames/tripletriad.zip"
        install_wine_bottle vb6run dx8vb speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/Triple Triad" "${cache}/tripletriad.zip"
        wine "c:\Program Files\Triple Triad\Checkup.exe" /verysilent
        add_launcher "c:\Program Files\Triple Triad\TripleTriad.exe"
    ;;
    "Troopanum2")
        download "https://www.agarchive.net/games/bsc/Troopanum2Setup.exe" "https://www.agarchive.net/games/bsc/BSC%20unlock%20code%20generator.7z"
        install_wine_bottle vb6run dx8vb
        # FIXME: Hacky, but it works. Install dotnet35 by itself so it actually doesn't hang.
        winetricks -q dotnet35sp1
        wineserver -k # Damn you, dotnet.
        wine "${cache}/Troopanum2Setup.exe" /silent &
        xdotool sleep 10 key --clearmodifiers alt+y 2> /dev/null
        sleep 30 && wineserver -k # Sometimes the installer finishes but the wineserver has more processes that don't exit, so we can't depend on wineserver -w.
        mkdir -p "$WINEPREFIX/drive_c/Program Files/bsc-key-generator"
        7z x -o"$WINEPREFIX/drive_c/Program Files/bsc-key-generator" "${cache}/BSC unlock code generator.7z"
        echo "$USER"$'\n'"$(hostname)"$'\n'"none"$'\n'"US" > "$WINEPREFIX/drive_c/Program Files/Troopanum 2.0/config.dat"
        unix2dos "$WINEPREFIX/drive_c/Program Files/Troopanum 2.0/config.dat"
        if command -v xclip &> /dev/null && command -v xdotool &> /dev/null ; then
            wine "c:\Program Files\Troopanum 2.0\register.exe" &
            xdotool sleep 10 key Return sleep 2 key Return sleep 2 key Alt+n
            sleep 1
            regcode="$(xclip -selection clipboard -o)"
            xdotool sleep 1 key Shift+Tab sleep 1 key Return
            # FIXME: Kind of hacky, but let's make sure it actually exitted.
            sleep 5
            wineserver -k
            wine "c:\Program Files\bsc-key-generator\BlindsoftwareUnlockCodeGenerator.exe" &
            xdotool sleep 10 key Return sleep 2 type t
            xdotool sleep 1 type t
            xdotool sleep 1 key Tab sleep 1 type $regcode
            xdotool sleep 1 key Tab sleep 1 key Return
            sleep 2
            regcode="$(xclip -selection clipboard -o)" # Might as well reuse the variable.
                        # FIXME: Kind of hacky, but let's make sure it actually exitted since I can't find a good way to exit this program.
            sleep 5
            wineserver -k
                     wine "c:\Program Files\Troopanum 2.0\register.exe" &
            echo "$regcode" | xclip -selection clipboard
            xdotool sleep 10 key Return sleep 2 key Shift+Tab sleep 1 key Shift+Tab sleep 1 key Return
            # FIXME: Kind of hacky, but let's make sure it actually exitted.
            sleep 5
            wineserver -k
        else
            echo "Warning: You need xclip and xdotool for this installer to finish the registration process, however, you don't have them. Either remove the program, install the stated tools, and reinstall it, or go on your own. Caveat Emptor, go to jail, do not pass go, do not collect audiogame rewards and all that stuff."
        fi
        add_launcher "c:\Program Files\Troopanum 2.0\troop.exe"
    ;;
    "Tube Sim")
        export winVer="win7"
        download "http://www.ndadamson.com/downloads/TubeSim1_1_Install.exe"
        install_wine_bottle speechsdk
        wine "${cache}/TubeSim1_1_Install.exe" /silent &
        xdotool sleep 10 key --clearmodifiers Return sleep 1 key alt+n sleep 1 key alt+a sleep 1 key alt+i sleep 10 key alt+f 2> /dev/null
        wineserver -w
        add_launcher "c:\Program Files\NASoft\TubeSim\tsMain.exe"
    ;;
    "Ultimate SounDoku")
        export winVer="win7"
        export bottle="draconis"
        download "http://download.dracoent.com/Windows/classic/USSetup.exe"
        install_wine_bottle vb6run dx8vb speechsdk quartz
        cp -v "${cache}/USSetup.exe" "$WINEPREFIX/drive_c/windows/temp/"
        wine "c:/windows/temp/USSetup.exe" /sp- /silent
        rm -fv "$WINEPREFIX/drive_c/windows/temp/USSetup.exe"
        # warning warning warning: Do not change location, or installer will not function.
        # FIXME: Hacky, but it works. Install dotnet20 by itself so it actually doesn't hang.
        winetricks -q dotnet20
        wineserver -k # Damn you, dotnet.
        add_launcher "c:\Program Files\Draconis Entertainment\Ultimate Soundoku\soundoku.exe"
    ;;
    "Undead Assault")
        export winVer="win7"
        download "http://undead-assault.com/static/files/public/undead_assault.zip" "${nvdaControllerClientDll}"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/undead_assault" "${cache}/undead_assault.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvdaControllerClient32.dll" "{}" \;
        add_launcher "c:\Program Files\undead_assault\Undead Assault.exe"
    ;;
    "Villains From Beyond")
        export bottle="oriol-gomez"
        export winVer="win7"
        download "${ipfsGateway}/ipfs/QmWx271xuk3Mv9XTBoVu5BDJvXFZdasawC2nhtV21WAaUU?filename=villains_en.zip"
        install_wine_bottle speechsdk
        unzip -d "$WINEPREFIX/drive_c/Program Files/villains from beyond" "${cache}/villains_en.zip"
        find "${WINEPREFIX}" -type f -name "nvdaControllerClient32.dll" -exec rm -fv "{}" \;
        add_launcher "c:\Program Files\villains from beyond\game.exe"
    ;;
    "VIP Mud")
        download "http://gmagames.com/vipmud20016.exe" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd32.dll"
        export winVer="win7"
        install_wine_bottle vb6run dx8vb speechsdk
        wine "${cache}/vipmud20016.exe" /silent
        mkdir -p "${HOME}/.local/wine/vip-mud/drive_c/users/${USER}/Documents/VIP Mud"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient32.dll' -exec cp -v "${cache}/nvda2speechd32.dll" "{}" \;
        add_launcher "c:\Program Files\VIPMud 2.0\vipmud2.exe"
        echo
        echo "When you first launch VIP Mud You will be presented with several inaccessible dialogs."
        echo "To bypass these dialogs, press alt+o, then press enter until speech starts."
        echo "To be sure that each new dialog has had time to complete, wait a second between each press of enter."
        echo "In each subsequent launch, you will need to hit enter a couple times before it starts speaking."
        alert
    ;;
    "Warsim")
        get_installer "Warsim Full Game.zip" "https://huw2k8.itch.io/warsim"
export WINEARCH=win64
        export winVer="win7"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files/Warsim/" "${cache}/Warsim Full Game.zip"
        add_launcher "c:\Program Files\Warsim\Warsim.exe"
    ;;
    "Wave of the Undead")
        export WINEARCH=win64
        export winVer="win7"
        download "https://dl.dropbox.com/scl/fi/ukvou0y4gwg21nhhdpj40/Wave-of-the-Undead-Setup.exe?rlkey=4xnuwicpmbkx6w2jo2i56mijg" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        install_wine_bottle
        wine "${cache}/Wave-of-the-Undead-Setup.exe" &
            xdotool sleep 20 key Alt+n sleep 2 key Alt+n sleep 2 key Alt+n
            xdotool sleep 2 key Alt+i
            xdotool sleep 20 key Alt+n
            xdotool sleep 2 key space
            xdotool sleep 2 key Alt+f
            wineserver -w
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files (x86)\Wave Of The Undead\wave_of_the_undead.exe"
    ;;
    "Wolf Games Launcher")
        download "https://sightlesswolf.com/wg-release.zip" "https://github.com/RastislavKish/nvda2speechd/releases/download/v0.1/nvda2speechd64.dll"
        export WINEARCH=win64
        export winVer="win7"
        install_wine_bottle
        unzip -d "$WINEPREFIX/drive_c/Program Files" "${cache}/wg-release.zip"
        find "${WINEPREFIX}" -type f -name 'nvdaControllerClient64.dll' -exec cp -v "${cache}/nvda2speechd64.dll" "{}" \;
        add_launcher "c:\Program Files\wg-release\WolfGames.exe"
    ;;
    "World of War")
        export bottle="nyanchan"
        export winVer="win7"
        download "https://www.agarchive.net/games/nyanchan/world%20of%20war%20English.7z"
        install_wine_bottle
        7z x -o"$WINEPREFIX/drive_c/nyanchangame" "${cache}/world of war English.7z"
        add_launcher "c:\nyanchangame\world of war English\world of war.exe"
    ;;
    "Donate")
        open_url "https://ko-fi.com/stormux"
    ;;
    "Become a Patron")
        open_url "https://2mb.games/product/2mb-patron/"
    ;;
    *)
        [[ "$agmNoLaunch" != "true" ]] && [[ -n "${game}" ]] && echo "Game \"${game}\" not found."
        [[ "$agmNoLaunch" != "true" ]] && exit 1
    ;;
esac
[[ "$agmNoLaunch" != "true" ]] && winetricks sandbox

[[ "$agmNoLaunch" != "true" ]] && exit 0
